@@ -598,6 +598,11 @@ let gFolderTreeView = {
     let count = dt.mozItemCount;
     let cs = MailServices.copy;
 
+    // This is a potential rss feed.  A link image as well as link text url
+    // should be handled; try to extract a url from non moz apps as well.
+    let feedUri = targetFolder.server.type == "rss" && count == 1 ?
+                    FeedUtils.getFeedUriFromDataTransfer(dt) : null;
+
     // we only support drag of a single flavor at a time.
     let types = dt.mozTypesAt(0);
     if (Array.indexOf(types, "text/x-moz-folder") != -1) {
@@ -658,6 +663,11 @@ let gFolderTreeView = {
       cs.CopyMessages(sourceFolder, array, targetFolder, isMove, null,
                         msgWindow, true);
     }
+    else if (feedUri) {
+      Cc["@mozilla.org/newsblog-feed-downloader;1"]
+         .getService(Ci.nsINewsBlogFeedDownloader)
+         .subscribeToFeed(feedUri.spec, targetFolder, msgWindow);
+    }
     else if (Array.indexOf(types, "application/x-moz-file") != -1) {
       for (let i = 0; i < count; i++) {
         let extFile = dt.mozGetDataAt("application/x-moz-file", i)
@@ -669,17 +679,6 @@ let gFolderTreeView = {
         }
       }
     }
-
-    if (targetFolder.server.type == "rss" && count == 1) {
-      // This is a potential rss feed.  A link image as well as link text url
-      // should be handled; try to extract a url from non moz apps as well.
-      let validUri = FeedUtils.getFeedUriFromDataTransfer(dt);
-
-      if (validUri)
-        Cc["@mozilla.org/newsblog-feed-downloader;1"]
-           .getService(Ci.nsINewsBlogFeedDownloader)
-           .subscribeToFeed(validUri.spec, targetFolder, msgWindow);
-    }
   },
 
   _onDragStart: function ftv_dragStart(aEvent) {
@@ -1290,42 +1289,11 @@ let gFolderTreeView = {
       generateMap: function ftv_recent_generateMap(ftv) {
         const MAXRECENT = 15;
 
-        /**
-         * Sorts our folders by their recent-times.
-         */
-        function sorter(a, b) {
-          return Number(a.getStringProperty("MRUTime")) <
-            Number(b.getStringProperty("MRUTime"));
-        }
-
-        /**
-         * This function will add a folder to the recentFolders array if it
-         * is among the 15 most recent.  If we exceed 15 folders, it will pop
-         * the oldest folder, ensuring that we end up with the right number
-         *
-         * @param aFolder the folder to check
-         */
-        let recentFolders = [];
-        let oldestTime = 0;
-        function addIfRecent(aFolder) {
-          let time;
-          try {
-            time = Number(aFolder.getStringProperty("MRUTime")) || 0;
-          } catch (ex) {return;}
-          if (time <= oldestTime)
-            return;
-
-          if (recentFolders.length == MAXRECENT) {
-            recentFolders.sort(sorter);
-            recentFolders.pop();
-            let oldestFolder = recentFolders[recentFolders.length - 1];
-            oldestTime = Number(oldestFolder.getStringProperty("MRUTime"));
-          }
-          recentFolders.push(aFolder);
-        }
-
-        for each (let folder in ftv._enumerateFolders)
-          addIfRecent(folder);
+        // Get 15 (MAXRECENT) most recently accessed folders.
+        let recentFolders = getMostRecentFolders(ftv._enumerateFolders,
+                                                 MAXRECENT,
+                                                 "MRUTime",
+                                                 null);
 
         // Sort the folder names alphabetically.
         recentFolders.sort(function rf_sort(a, b){
@@ -1335,15 +1303,15 @@ let gFolderTreeView = {
             aLabel = a.server.prettyName;
             bLabel = b.server.prettyName;
           }
-          return aLabel.toLocaleLowerCase() > bLabel.toLocaleLowerCase();
+          return aLabel.localeCompare(bLabel);
         });
 
-        let items = [new ftvItem(f) for each (f in recentFolders)];
+        let items = [new ftvItem(f) for (f of recentFolders)];
 
         // There are no children in this view!
         // And we want to display the account name to distinguish folders w/
         // the same name.
-        for each (let folder in items) {
+        for (let folder of items) {
           folder.__defineGetter__("children", function() []);
           folder.addServerName = true;
         }
@@ -1886,7 +1854,7 @@ ftvItem.prototype = {
         text += " - " + this._folder.server.prettyName;
     }
     // Yeah, we hard-code this, but so did the old code...
-    let unread = this._folder.getNumUnread(false);
+    let unread = this._folder.getNumUnread(!this.open);
     if (unread > 0)
       text += " (" + unread + ")";
     return text;
@@ -1910,6 +1878,10 @@ ftvItem.prototype = {
       properties += " specialFolder-" + smartFolderName.replace(' ','');
     }
 
+    if (this._folder.server.type == "rss" && !this._folder.isServer &&
+        FeedUtils.getFeedUrlsInFolder(this._folder))
+      properties += " isFeedFolder-true";
+
     return properties;
   },
 
@@ -1973,6 +1945,10 @@ let gFolderTreeController = {
       dualUseFolders = folder.server.dualUseFolders;
 
     function newFolderCallback(aName, aFolder) {
+      // createSubfolder can throw an exception, causing the newFolder dialog
+      // to not close and wait for another input.
+      // TODO: Rewrite this logic and also move the opening of alert dialogs from
+      // nsMsgLocalMailFolder::CreateSubfolderInternal to here (bug 831190#c16).
       if (aName)
         aFolder.createSubfolder(aName, msgWindow);
     }
@@ -2348,7 +2324,7 @@ function sortFolderItems (aFtvItems) {
     let sortKey = a._folder.compareSortKeys(b._folder);
     if (sortKey)
       return sortKey;
-    return a.text.toLowerCase() > b.text.toLowerCase();
+    return a.text.localeCompare(b.text);
   }
   aFtvItems.sort(sorter);
 }
