@@ -15,6 +15,7 @@ Components.utils.import("resource:///modules/mailServices.js");
 Components.utils.import("resource:///modules/msgDBCacheManager.js");
 Components.utils.import("resource:///modules/sessionStoreManager.js");
 Components.utils.import("resource:///modules/summaryFrameManager.js");
+Components.utils.import("resource:///modules/MailUtils.js");
 Components.utils.import("resource://gre/modules/Services.jsm");
 
 /* This is where functions related to the 3 pane window are kept */
@@ -106,6 +107,64 @@ var folderListener = {
     }
 }
 
+/*
+ * Listen for Lightweight Theme styling changes and update the theme accordingly.
+ */
+let LightweightThemeListener = {
+  _modifiedStyles: [],
+
+  init: function () {
+    XPCOMUtils.defineLazyGetter(this, "styleSheet", function() {
+      for (let i = document.styleSheets.length - 1; i >= 0; i--) {
+        let sheet = document.styleSheets[i];
+        if (sheet.href == "chrome://messenger/skin/messengerLWTheme.css")
+          return sheet;
+      }
+    });
+
+    Services.obs.addObserver(this, "lightweight-theme-styling-update", false);
+    if (document.documentElement.hasAttribute("lwtheme"))
+      this.updateStyleSheet(document.documentElement.style.backgroundImage);
+  },
+
+  uninit: function () {
+    Services.obs.removeObserver(this, "lightweight-theme-styling-update");
+  },
+
+  /**
+   * Append the headerImage to the background-image property of all rulesets in
+   * messengerLWTheme.css.
+   *
+   * @param headerImage - a string containing a CSS image for the lightweight
+   * theme header.
+   */
+  updateStyleSheet: function(headerImage) {
+    if (!this.styleSheet)
+      return;
+    for (let i = 0; i < this.styleSheet.cssRules.length; i++) {
+      let rule = this.styleSheet.cssRules[i];
+      if (!rule.style.backgroundImage)
+        continue;
+
+      if (!this._modifiedStyles[i])
+        this._modifiedStyles[i] = { backgroundImage: rule.style.backgroundImage };
+
+      rule.style.backgroundImage = this._modifiedStyles[i].backgroundImage + ", " + headerImage;
+    }
+  },
+
+  // nsIObserver
+  observe: function (aSubject, aTopic, aData) {
+    if (aTopic != "lightweight-theme-styling-update" || !this.styleSheet)
+      return;
+
+    let themeData = JSON.parse(aData);
+    if (!themeData)
+      return;
+    this.updateStyleSheet("url(" + themeData.headerURL + ")");
+  },
+};
+
 function ServerContainsFolder(server, folder)
 {
   if (!folder || !server)
@@ -334,6 +393,8 @@ function OnLoadMessenger()
   migrateMailnews();
   // Rig up our TabsInTitlebar early so that we can catch any resize events.
   TabsInTitlebar.init();
+  // Listen for Lightweight Theme styling changes and update the theme accordingly.
+  LightweightThemeListener.init();
   // update the pane config before we exit onload otherwise the user may see a flicker if we poke the document
   // in delayedOnLoadMessenger...
   UpdateMailPaneConfig(false);
@@ -600,6 +661,8 @@ function OnUnloadMessenger()
 
   TabsInTitlebar.uninit();
 
+  LightweightThemeListener.uninit();
+
   let tabmail = document.getElementById("tabmail");
   tabmail._teardown();
 
@@ -756,7 +819,7 @@ function loadStartFolder(initialUri)
     {
 
         if(initialUri)
-            startFolder = GetMsgFolderFromUri(initialUri);
+            startFolder = MailUtils.getFolderForURI(initialUri);
         else
         {
             try {
@@ -1168,7 +1231,7 @@ function GetSelectedMsgFolders()
 
 function SelectFolder(folderUri)
 {
-  gFolderTreeView.selectFolder(GetMsgFolderFromUri(folderUri));
+  gFolderTreeView.selectFolder(MailUtils.getFolderForURI(folderUri));
 }
 
 function ReloadMessage()
@@ -1299,25 +1362,34 @@ function ThreadPaneOnDragStart(aEvent) {
     return;
 
   gFolderDisplay.hintAboutToDeleteMessages();
-  let fileNames = [];
+  let fileNames = new Set();
   let msgUrls = {};
 
-  // dragging multiple messages to desktop does not
-  // currently work, pending core fixes for
-  // multiple-drop-on-desktop support. (bug 513464)
+  // Dragging multiple messages to desktop does not currently work.
+  // When core fixes for multiple-drop-on-desktop support
+  // (e.g. bug 513464, bug 270292) are landed, generating of
+  // "application/x-moz-file-promise" values for i > 0 can be enabled.
+  // But first ensure suggestUniqueFileName is efficient enough on 10000+ dragged
+  // messages.
   for (let i in messages) {
     messenger.messageServiceFromURI(messages[i])
              .GetUrlForUri(messages[i], msgUrls, null);
-    var subject = messenger.messageServiceFromURI(messages[i])
-                           .messageURIToMsgHdr(messages[i]).mime2DecodedSubject;
-    var uniqueFileName = suggestUniqueFileName(subject.substr(0,124), ".eml",
-                                               fileNames);
-    fileNames[i] = uniqueFileName;
     aEvent.dataTransfer.mozSetDataAt("text/x-moz-message", messages[i], i);
     aEvent.dataTransfer.mozSetDataAt("text/x-moz-url",msgUrls.value.spec, i);
+
+    if (i > 0)
+      continue;
+
+    // Generate file name in case the object is dropped onto the desktop.
+    let subject = messenger.messageServiceFromURI(messages[i])
+                           .messageURIToMsgHdr(messages[i]).mime2DecodedSubject;
+    let uniqueFileName = suggestUniqueFileName(subject.substr(0, 124), ".eml",
+                                               fileNames);
+    fileNames.add(uniqueFileName);
+
     aEvent.dataTransfer.mozSetDataAt("application/x-moz-file-promise-url",
-                                     msgUrls.value.spec + "?fileName=" + uniqueFileName,
-                                     i);
+                                     msgUrls.value.spec + "?fileName=" +
+                                     uniqueFileName, i);
     aEvent.dataTransfer.mozSetDataAt("application/x-moz-file-promise", null, i);
   }
   aEvent.dataTransfer.effectAllowed = "copyMove";
@@ -1325,29 +1397,30 @@ function ThreadPaneOnDragStart(aEvent) {
 }
 
 /**
- * example use:
- *   suggestUniqueFileName("testname",".txt",["testname", "testname1"])
- *   returns "testname2"
- * does not check file system for existing files
- * @param existingNames array of names in use
+ * Returns a new filename that is guaranteed to not be in the Set
+ * of existing names.
+ *
+ * Example use:
+ *   suggestUniqueFileName("testname", ".txt", Set("testname", "testname1"))
+ *   returns "testname2.txt"
+ * Does not check file system for existing files.
+ *
+ * @param aIdentifier     proposed filename
+ * @param aType           extension
+ * @param aExistingNames  a Set of names already in use
  */
-function suggestUniqueFileName(identifier, type, existingNames) {
+function suggestUniqueFileName(aIdentifier, aType, aExistingNames) {
   let suffix = 1;
-  let suggestion;
-  let base = identifier;
-  let exists;
-  do {
-    exists = false;
-    suggestion = GenerateValidFilename(base, type);
-    for (let i = 0; i < existingNames.length; i++) {
-      if (existingNames[i] == suggestion) {
-        base = identifier + suffix;
-        suffix++;
-        exists = true;
-        break;
-      }
-    }
-  } while (exists);
+  let base = validateFileName(aIdentifier);
+  let suggestion = base + aType;
+  while(true) {
+    if (!aExistingNames.has(suggestion))
+      break;
+
+    suggestion = base + suffix + aType;
+    suffix++;
+  }
+
   return suggestion;
 }
 
