@@ -13,7 +13,7 @@ let gSelectionSummaryStrings = {
   NConversations: "NConversations",
   numMsgs: "numMsgs",
   countUnread: "countUnread",
-  Nmessages: "Nmessages",
+  ignoredCount: "ignoredCount",
   messagesSize: "messagesSize",
   noticeText: "noticeText",
   noSubject: "noSubject",
@@ -32,56 +32,25 @@ function loadSelectionSummaryStrings() {
   gSelectionSummaryStringsInitialized = true;
 
   // convert strings to those in the string bundle
-  let getStr = function(string) document.getElementById("bundle_multimessages").getString(string);
+  let getStr = function(string) {
+    return document.getElementById("bundle_multimessages").getString(string);
+  };
   for (let [name, value] in Iterator(gSelectionSummaryStrings))
     gSelectionSummaryStrings[name] = typeof value == "string" ?
       getStr(value) : value.map(gSelectionSummaryStrings);
 }
 
-// Ah, wouldn't it be nice if there was platform code to do the following...
-
-/**
- * Adds a classes to a node. Avoids duplicates, nothing fancy.
- *
- * @param node
- *        any old DOM node
- * @param classArray
- *        an array of strings, which will be added as CSS classes
- */
-function _mm_addClass(node, classArray) {
-  for (let i = 0; i < classArray.length; i++)
-  {
-    if (!node.classList.contains(classArray[i]))
-      node.classList.add(classArray[i]);
-  }
-}
-
-/**
- * the equivalent of jQuery's removeClass.  Doesn't freak if the class name
- * isn't in the class attribute.
- *
- * @param node
- *        any old DOM node
- * @param classname
- *        a string, which will be removed from the class set.
- */
-function _mm_removeClass(node, classname) {
-  if (node.classList.contains(classname))
-    node.classList.remove(classname);
-}
-
 /**
  * Format the display name for the multi-message/thread summaries. First, try
  * using FormatDisplayName, then fall back to the header's display name or the
  * address.
  *
- * @param aHeaderParser An instance of |nsIMsgHeaderParser|
- * @param aHeaderValue  The raw header value
- * @param aContext      The context of the header field (e.g. "to", "from")
- * @return The formatted display name
+ * @param aHeaderParser An instance of |nsIMsgHeaderParser|.
+ * @param aHeaderValue  The raw header value.
+ * @param aContext      The context of the header field (e.g. "to", "from").
+ * @return The formatted display name.
  */
-function _mm_FormatDisplayName(aHeaderParser, aHeaderValue, aContext)
-{
+function _mm_FormatDisplayName(aHeaderParser, aHeaderValue, aContext) {
   let addresses = {};
   let fullNames = {};
   let names = {};
@@ -99,31 +68,28 @@ function _mm_FormatDisplayName(aHeaderParser, aHeaderValue, aContext)
 }
 
 function _mm_escapeHTML(aUnescaped) {
-  return aUnescaped.replace('&', "&amp;", 'g')
-                   .replace('<', "&lt;", 'g')
-                   .replace('>', "&gt;", 'g');
+  return aUnescaped.replace("&", "&amp;", "g")
+                   .replace("<", "&lt;", "g")
+                   .replace(">", "&gt;", "g");
 }
 
 /**
- * the MultiMessageSummary class is responsible for populating the message pane
+ * The MultiMessageSummary class is responsible for populating the message pane
  * with a reasonable summary of a set of messages that span more than one
  * thread.
  *
- * It uses the same multimessage iframe as ThreadSummary, so both it
- * and ThreadSummary should be careful to clean up the other's work
- * before inserting their DOM nodes into the frame.
+ * It uses the same multimessage iframe as ThreadSummary, so both it and
+ * ThreadSummary should be careful to clean up the other's work before
+ * inserting their DOM nodes into the frame.
  *
  * There's a two phase process: build the framework based on what's available
  * from the msgHdr itself, and then spawn an aysnc Gloda query which will
  * fetch the snippets, tags, etc.
  *
- * @param aMessages
- *        Array of message headers.
- * @param [aListener]
- *        An optional listener that implements onLoadStarted and
- *        onLoadCompleted.
+ * @param aMessages   Array of message headers.
+ * @param [aListener] An optional listener that implements onLoadStarted and
+ *                    onLoadCompleted.
  */
-
 function MultiMessageSummary(aMessages, aListener) {
   this._msgHdrs = aMessages;
   this._listener = aListener;
@@ -143,17 +109,16 @@ MultiMessageSummary.prototype = {
   },
 
   /**
-   * Given a msgHdr, return a list of tag objects. This function
-   * just does the messy work of understanding how tags are
-   * stored in nsIMsgDBHdrs.  It would be a good candidate for
-   * a utility library.
+   * Given a msgHdr, return a list of tag objects. This function just does the
+   * messy work of understanding how tags are stored in nsIMsgDBHdrs.  It would
+   * be a good candidate for a utility library.
    *
-   * @param aMsgHdr: the msgHdr whose tags we want
-   * @return a list of tag objects.
+   * @param aMsgHdr The msgHdr whose tags we want.
+   * @return A list of tag objects.
    */
   getTagsForMsg: function(aMsgHdr) {
     let keywords = aMsgHdr.getStringProperty("keywords");
-    let keywordList = keywords.split(' ');
+    let keywordList = keywords.split(" ");
     let keywordMap = {};
     for (let iKeyword = 0; iKeyword < keywordList.length; iKeyword++) {
       let keyword = keywordList[iKeyword];
@@ -193,20 +158,22 @@ MultiMessageSummary.prototype = {
 
   /**
    * Fill in the summary pane describing the selected messages
-   **/
+   */
   _onLoad: function() {
-    let htmlpane = document.getElementById('multimessage');
-    // First, we group the messages in threads.
-    // count threads
-    let threads = {};
-    let numThreads = 0;
-    let viewThreadId = function (aMsgHdr) {
+    function viewThreadId(aMsgHdr) {
       let thread = gDBView.getThreadContainingMsgHdr(aMsgHdr);
       return thread.threadKey;
-    };
+    }
+
+    let htmlpane = document.getElementById("multimessage");
+    let contentDocument = htmlpane.contentDocument;
+
+    // First, we group the messages in threads and count the threads.
+    let threads = {};
+    let numThreads = 0;
     for (let [,msgHdr] in Iterator(this._msgHdrs))
     {
-      if (! threads[viewThreadId(msgHdr)]) {
+      if (!threads[viewThreadId(msgHdr)]) {
         threads[viewThreadId(msgHdr)] = [msgHdr];
         numThreads += 1;
       } else {
@@ -214,23 +181,22 @@ MultiMessageSummary.prototype = {
       }
     }
 
-    // set the heading based on the number of messages & threads
-    let heading = htmlpane.contentDocument.getElementById('heading');
-    _mm_addClass(heading, ["heading"]);
-    _mm_addClass(heading, ["info"]);
+    // Set the heading based on the number of messages & threads.
+    let heading = contentDocument.getElementById("heading");
+    heading.classList.add("heading", "info");
 
-    let messagesTitle =
-      PluralForm.get(numThreads, gSelectionSummaryStrings["NConversations"])
-                .replace("#1", numThreads);
+    let messagesTitle = PluralForm.get(
+      numThreads, gSelectionSummaryStrings["NConversations"]
+    ).replace("#1", numThreads);
 
     heading.textContent = messagesTitle;
 
-    // enable/disable the archive button as appropriate
-    let archiveBtn = htmlpane.contentDocument.getElementById('hdrArchiveButton');
+    // Enable/disable the archive button as appropriate.
+    let archiveBtn = contentDocument.getElementById("hdrArchiveButton");
     archiveBtn.collapsed = !gFolderDisplay.canArchiveSelectedMessages;
 
-    // clear the messages list
-    let messagesElt = htmlpane.contentDocument.getElementById('messagelist');
+    // Clear the messages list.
+    let messagesElt = contentDocument.getElementById("messagelist");
     while (messagesElt.firstChild)
       messagesElt.removeChild(messagesElt.firstChild);
 
@@ -240,7 +206,7 @@ MultiMessageSummary.prototype = {
     let maxCountExceeded = false;
     var parser = new DOMParser();
 
-    for (let [thread,msgs] in Iterator(threads)) {
+    for (let [thread, msgs] in Iterator(threads)) {
       count += msgs.length;
       if (count > MAX_MESSAGES) {
         maxCountExceeded = true;
@@ -250,32 +216,33 @@ MultiMessageSummary.prototype = {
       let countStarred = 0;
       let header, countNode;
 
-      // we'll mark the thread unread if any messages in it are unread
-      for (let [, msgHdr] in Iterator(msgs)) {
-        if (! msgHdr.isRead)
-          countUnread += 1;
+      // We'll mark the thread unread if any messages in it are unread.
+      for (let [,msgHdr] in Iterator(msgs)) {
+        if (!msgHdr.isRead)
+          countUnread++;
         if (msgHdr.isFlagged)
-          countStarred += 1;
+          countStarred++;
       }
 
       let numMsgs = msgs.length;
-      let msg_classes = ["message"];
-      if (numMsgs > 1)
-        msg_classes.push("thread");
-      if (countUnread)
-        msg_classes.push("unread");
-      if (countStarred)
-        msg_classes.push("starred");
-
-      let subject = msgs[0].mime2DecodedSubject || gSelectionSummaryStrings['noSubject'];
-      let author = _mm_FormatDisplayName(MailServices.headerParser, msgs[0].mime2DecodedAuthor, "from");
+      let subject = msgs[0].mime2DecodedSubject ||
+                    gSelectionSummaryStrings["noSubject"];
+      let author = _mm_FormatDisplayName(
+        MailServices.headerParser, msgs[0].mime2DecodedAuthor, "from"
+      );
 
       let countstring = "";
       if (numMsgs > 1) {
         countstring += "(";
-        countstring += PluralForm.get(numMsgs, gSelectionSummaryStrings["numMsgs"]).replace('#1', numMsgs);
-        if (countUnread)
-          countstring += PluralForm.get(countUnread, gSelectionSummaryStrings["countUnread"]).replace('#1', countUnread);
+        countstring += PluralForm.get(
+          numMsgs, gSelectionSummaryStrings["numMsgs"]
+        ).replace("#1", numMsgs);
+
+        if (countUnread) {
+          countstring += PluralForm.get(
+            countUnread, gSelectionSummaryStrings["countUnread"]
+          ).replace("#1", countUnread);
+        }
         countstring += ")";
       }
 
@@ -294,11 +261,19 @@ MultiMessageSummary.prototype = {
                         '  </div>' +
                         '</div>';
 
-      let msgNode = htmlpane.contentDocument.createElement("div");
+      let msgNode = contentDocument.createElement("div");
       // innerHTML is safe here because all of the data in msgContents is
       // either generated from integers or escaped to be safe.
       msgNode.innerHTML = msgContents;
-      _mm_addClass(msgNode, msg_classes);
+
+      msgNode.classList.add("message");
+      if (numMsgs > 1)
+        msgNode.classList.add("thread");
+      if (countUnread)
+        msgNode.classList.add("unread");
+      if (countStarred)
+        msgNode.classList.add("starred");
+
       messagesElt.appendChild(msgNode);
 
       let snippetNode = msgNode.querySelector(".snippet");
@@ -332,73 +307,82 @@ MultiMessageSummary.prototype = {
       while (tagsNode.firstChild)
         tagsNode.removeChild(tagsNode.firstChild);
       this._addTagNodes(msgs, tagsNode);
-      for (let [, msgHdr] in Iterator(msgs)) {
+      for (let [,msgHdr] in Iterator(msgs)) {
         this._msgNodes[msgHdr.messageKey + msgHdr.folder.URI] = msgNode;
       }
       messagesElt.appendChild(msgNode);
     }
-    this.computeSize(htmlpane);
-    this.notifyMaxCountExceeded(htmlpane.contentDocument, this._msgHdrs.length, MAX_MESSAGES);
+    this.computeSize(contentDocument);
+    this.notifyMaxCountExceeded(
+      contentDocument, this._msgHdrs.length, MAX_MESSAGES
+    );
 
-    this._glodaQueries.push(Gloda.getMessageCollectionForHeaders(this._msgHdrs, this));
-    htmlpane.contentDocument.defaultView.adjustHeadingSize();
+    this._glodaQueries.push(Gloda.getMessageCollectionForHeaders(
+      this._msgHdrs, this
+    ));
+    contentDocument.defaultView.adjustHeadingSize();
   },
 
   /**
-   * clear out the tagsnode, and fill in appropriately for the union of
-   * tags in the specified messags
+   * Clear out the tagsnode, and fill in appropriately for the union of tags in
+   * the specified messags.
    */
   _addTagNodes: function(msgs, tagsNode) {
-      // for tags, stars, and read/unread status, we want to map
-      // from all messages to one node
-      let tags = {};
-      for each (let [, msgHdr] in Iterator(msgs)) {
-        let msgTags = this.getTagsForMsg(msgHdr);
-        for each (let [,tag] in Iterator(msgTags)) {
-          if (!(tag.key in tags)) {
-            tags[tag.key] = tag;
-          }
+    // For tags, stars, and read/unread status, we want to map from all
+    // messages to one node.
+    let tags = {};
+    for (let msgHdr of msgs) {
+      for (let tag of this.getTagsForMsg(msgHdr)) {
+        if (!(tag.key in tags)) {
+          tags[tag.key] = tag;
         }
       }
-      for each (let [, tag] in Iterator(tags)) {
-        let tagNode = tagsNode.ownerDocument.createElement('span');
-        // see tagColors.css
-        let colorClass = "blc-" + this._msgTagService.getColorForKey(tag.key).substr(1);
-        _mm_addClass(tagNode, ["tag", tag.tag, colorClass]);
-        tagNode.textContent = tag.tag;
-        tagsNode.appendChild(tagNode);
-      }
+    }
+    for (let [,tag] in Iterator(tags)) {
+      let tagNode = tagsNode.ownerDocument.createElement("span");
+      // See tagColors.css.
+      let colorClass = "blc-" + this._msgTagService
+                                    .getColorForKey(tag.key)
+                                    .substr(1);
+      tagNode.classList.add("tag", colorClass);
+      tagNode.dataset.tag = tag.tag;
+      tagNode.textContent = tag.tag;
+      tagsNode.appendChild(tagNode);
+    }
   },
 
   /**
-   * compute the size of the messages in the selection and display it
-   * in the element of id "size"
-  **/
-  computeSize: function(htmlpane) {
+   * Compute the size of the messages in the selection and display it in the
+   * element of id "size".
+   */
+  computeSize: function(contentDocument) {
     let numThreads = 0;
     let numBytes = 0;
 
     for (let [,msgHdr] in Iterator(this._msgHdrs))
       numBytes += msgHdr.messageSize; // XXX do something about news?
     let [size, unit] = DownloadUtils.convertByteUnits(numBytes);
-    let sizeText = replaceInsert(gSelectionSummaryStrings.messagesSize, 1, size);
+    let sizeText = replaceInsert(
+      gSelectionSummaryStrings.messagesSize, 1, size
+    );
     sizeText = replaceInsert(sizeText, 2, unit);
-    htmlpane.contentDocument.getElementById('size').textContent = sizeText;
+    contentDocument.getElementById("size").textContent = sizeText;
   },
 
-  /** Indicate if we're not summarizing _all_ of the specified messages because
+  /**
+   * Indicate if we're not summarizing _all_ of the specified messages because
    * that'd just be too much.
-  **/
+   */
   notifyMaxCountExceeded: function(aContentDocument, aNumMessages, aMaxCount) {
-    let notice = aContentDocument.getElementById('notice');
+    let notice = aContentDocument.getElementById("notice");
     if (aNumMessages > aMaxCount) {
       let noticeText = gSelectionSummaryStrings.noticeText;
       noticeText = replaceInsert(noticeText, 1, aNumMessages);
       noticeText = replaceInsert(noticeText, 2, aMaxCount);
       notice.textContent = noticeText;
-      _mm_removeClass(notice, 'hidden');
+      notice.classList.remove("hidden");
     } else {
-      _mm_addClass(notice, ['hidden']);
+      notice.classList.add("hidden");
     }
   },
 
@@ -415,9 +399,8 @@ MultiMessageSummary.prototype = {
    * Given a set of items from a gloda collection, process them and update
    * the display accordingly.
    *
-   * @param aItems
-   *        contents of a gloda collection
-  **/
+   * @param aItems Contents of a gloda collection.
+   */
   processItems: function(aItems) {
     let knownMessageNodes = [];
 
@@ -425,58 +408,50 @@ MultiMessageSummary.prototype = {
       let messageKey = glodaMsg.messageKey;
       let domkey = messageKey + glodaMsg.folder.uri;
 
-      // Unread and starred will get set if any of the messages in a
-      // collapsed thread qualify
-      // The trick here is that we may get multiple items corresponding to the same
-      // thread (and hence DOM node), so we need to detect when we get the first
-      // item for a particular DOM node, stash the preexisting status of that DOM
-      // node, an only do transitions if the items warrant it.
+      // Unread and starred will get set if any of the messages in a collapsed
+      // thread qualify.  The trick here is that we may get multiple items
+      // corresponding to the same thread (and hence DOM node), so we need to
+      // detect when we get the first item for a particular DOM node, stash the
+      // preexisting status of that DOM node, an only do transitions if the
+      // items warrant it.
       let headerNode = this._msgNodes[domkey];
       if (! headerNode.flags) {
         headerNode.flags = {};
         knownMessageNodes.push(headerNode);
       }
 
-      if (! glodaMsg.read)
-        headerNode.flags['unread'] = true;
-      if (glodaMsg.starred)
-        headerNode.flags['starred'] = true;
+      headerNode.flags["unread"] = !glodaMsg.read;
+      headerNode.flags["starred"] = glodaMsg.starred;
 
-      // for tags, there's a minor problem in that if _some_ of the items in a
-      // thread got modified
+      // For tags, there's a minor problem in that if _some_ of the items in a
+      // thread got modified.
       let key = messageKey + glodaMsg.folder.uri;
-      let tagsNode = headerNode.querySelector('.tags');
+      let tagsNode = headerNode.querySelector(".tags");
       while (tagsNode.firstChild)
         tagsNode.removeChild(tagsNode.firstChild);
-      this._addTagNodes([msg.folderMessage for each ([i,msg] in Iterator(aItems))],
+      this._addTagNodes([msg.folderMessage for ([,msg] in Iterator(aItems))],
                         tagsNode);
     }
 
-    for ([, headerNode] in Iterator(knownMessageNodes)) {
-      if (headerNode.flags['unread'])
-        _mm_addClass(headerNode, ["unread"]);
-      else
-        _mm_removeClass(headerNode, "unread");
-      if (headerNode.flags['starred'])
-        _mm_addClass(headerNode, ["starred"]);
-      else
-        _mm_removeClass(headerNode, "starred");
+    for (let headerNode of knownMessageNodes) {
+      headerNode.classList.toggle("unread", headerNode.flags["unread"]);
+      headerNode.classList.toggle("starred", headerNode.flags["starred"]);
       headerNode.flags = null;
     }
   },
 
   onQueryCompleted: function(aCollection) {
-    /* if we need something that's just available from GlodaMessages,
-      this is where we'll get it initially */
+    // If we need something that's just available from GlodaMessages, this is
+    // where we'll get it initially.
     if (this._listener)
       this._listener.onLoadCompleted();
     return;
   }
-}
+};
 
 
 /**
- * the ThreadSummary class is responsible for populating the message pane
+ * The ThreadSummary class is responsible for populating the message pane
  * with a reasonable summary of a set of messages that are are in a single
  * thread.
  *
@@ -488,13 +463,10 @@ MultiMessageSummary.prototype = {
  * from the msgHdr itself, and then spawn an aysnc Gloda query which will
  * fetch the snippets, tags, etc.
  *
- * @param aMessages
- *        Array of message headers.
- * @param [aListener]
- *        An optional listener that implements onLoadStarted and
- *        onLoadCompleted.
+ * @param aMessages   Array of message headers.
+ * @param [aListener] An optional listener that implements onLoadStarted and
+ *                    onLoadCompleted.
  */
-
 function ThreadSummary(aMessages, aListener)
 {
   this._msgHdrs = aMessages;
@@ -510,44 +482,42 @@ ThreadSummary.prototype = {
   _onLoad: function() {
     this._msgNodes = {};
 
-    let htmlpane = document.getElementById('multimessage');
+    let htmlpane = document.getElementById("multimessage");
+    let contentDocument = htmlpane.contentDocument;
 
     let firstMsgHdr = this._msgHdrs[0];
     let numMessages = this._msgHdrs.length;
-    let subject = (firstMsgHdr.mime2DecodedSubject || gSelectionSummaryStrings["noSubject"])
-       + " "
-       + PluralForm.get(numMessages, gSelectionSummaryStrings["Nmessages"]).replace('#1', numMessages);
-    let heading = htmlpane.contentDocument.getElementById('heading');
-    heading.setAttribute("class", "heading");
-    heading.textContent = subject;
 
-    // enable/disable the archive button as appropriate
-    let archiveBtn = htmlpane.contentDocument.getElementById('hdrArchiveButton');
+    // Enable/disable the archive button as appropriate.
+    let archiveBtn = contentDocument.getElementById("hdrArchiveButton");
     archiveBtn.collapsed = !gFolderDisplay.canArchiveSelectedMessages;
 
-    let messagesElt = htmlpane.contentDocument.getElementById('messagelist');
+    let messagesElt = contentDocument.getElementById("messagelist");
     while (messagesElt.firstChild)
       messagesElt.removeChild(messagesElt.firstChild);
 
     let count = 0;
+    let ignoredCount = 0;
     const MAX_THREADS = 100;
     const SNIPPET_LENGTH = 300;
     let maxCountExceeded = false;
     for (let i = 0; i < numMessages; ++i) {
-      count += 1;
+      let msgHdr = this._msgHdrs[i];
+
+      if (msgHdr.isKilled) { // ignored subthread...
+        ignoredCount++;
+        continue;
+      }
+
+      count++;
       if (count > MAX_THREADS) {
         maxCountExceeded = true;
         break;
       }
-      let msgHdr = this._msgHdrs[i];
 
-      let msg_classes = ["message"];
-      if (! msgHdr.isRead)
-        msg_classes.push("unread");
-      if (msgHdr.isFlagged)
-        msg_classes.push("starred");
-
-      let senderName = _mm_FormatDisplayName(MailServices.headerParser, msgHdr.mime2DecodedAuthor, "from");
+      let senderName = _mm_FormatDisplayName(
+        MailServices.headerParser, msgHdr.mime2DecodedAuthor, "from"
+      );
       let date = makeFriendlyDateAgo(new Date(msgHdr.date/1000));
 
       let msgContents = '<div class="row">' +
@@ -567,7 +537,13 @@ ThreadSummary.prototype = {
       // innerHTML is safe here because all of the data in msgContents is
       // either generated from integers or escaped to be safe.
       msgNode.innerHTML = msgContents;
-      _mm_addClass(msgNode, msg_classes);
+
+      msgNode.classList.add("message");
+      if (!msgHdr.isRead)
+        msgNode.classList.add("unread");
+      if (msgHdr.isFlagged)
+        msgNode.classList.add("starred");
+
       messagesElt.appendChild(msgNode);
 
       let key = msgHdr.messageKey + msgHdr.folder.URI;
@@ -590,13 +566,16 @@ ThreadSummary.prototype = {
         // that's fixed, this code should adapt. XXX
         snippetNode.textContent = "...";
       }
+
       let tagsNode = msgNode.querySelector(".tags");
-      let tags = this.getTagsForMsg(msgHdr);
-      for each (let [,tag] in Iterator(tags)) {
-        let tagNode = tagsNode.ownerDocument.createElement('span');
-        // see tagColors.css
-        let colorClass = "blc-" + this._msgTagService.getColorForKey(tag.key).substr(1);
-        _mm_addClass(tagNode, ["tag", tag.tag, colorClass]);
+      for (let tag of this.getTagsForMsg(msgHdr)) {
+        let tagNode = tagsNode.ownerDocument.createElement("span");
+        // See tagColors.css.
+        let colorClass = "blc-" + this._msgTagService
+                                      .getColorForKey(tag.key)
+                                      .substr(1);
+        tagNode.classList.add("tag", colorClass);
+        tagNode.dataset.tag = tag.tag;
         tagNode.textContent = tag.tag;
         tagsNode.appendChild(tagNode);
       }
@@ -619,12 +598,31 @@ ThreadSummary.prototype = {
 
       messagesElt.appendChild(msgNode);
     }
-    // stash somewhere so it doesn't get GC'ed
-    this._glodaQueries.push(Gloda.getMessageCollectionForHeaders(this._msgHdrs, this));
-    this.notifyMaxCountExceeded(htmlpane.contentDocument, numMessages, MAX_THREADS);
 
-    this.computeSize(htmlpane);
-    htmlpane.contentDocument.defaultView.adjustHeadingSize();
+    let countInfo = PluralForm.get(
+      numMessages, gSelectionSummaryStrings["numMsgs"]
+    ).replace("#1", numMessages);
+    if (ignoredCount != 0) {
+      countInfo += " - " + PluralForm.get(
+        ignoredCount, gSelectionSummaryStrings["ignoredCount"]
+      ).replace("#1", ignoredCount);
+    }
+
+    let subject = (firstMsgHdr.mime2DecodedSubject ||
+                   gSelectionSummaryStrings["noSubject"]) +
+                  " (" + countInfo + ")";
+    let heading = contentDocument.getElementById("heading");
+    heading.setAttribute("class", "heading");
+    heading.textContent = subject;
+
+    // Stash somewhere so it doesn't get GC'ed.
+    this._glodaQueries.push(Gloda.getMessageCollectionForHeaders(
+      this._msgHdrs, this
+    ));
+    this.notifyMaxCountExceeded(contentDocument, numMessages, MAX_THREADS);
+
+    this.computeSize(contentDocument);
+    contentDocument.defaultView.adjustHeadingSize();
   }
 };
 
@@ -633,26 +631,55 @@ ThreadSummary.prototype = {
 // is gone before it returns.
 var gSummary;
 
+/**
+ * Summarize a set of selected messages.  This can either be a single thread or
+ * multiple threads.
+ *
+ * @param aMessageDisplay The MessageDisplayWidget object responsible for
+ *                         showing messages.
+ */
+function summarizeSelection(aMessageDisplay) {
+  // Figure out if we're looking at one thread or more than one thread.
+  let folderDisplay = aMessageDisplay.folderDisplay;
+  let selectedIndices = folderDisplay.selectedIndices;
+  let dbView = folderDisplay.view.dbView;
+
+  let getThreadId = function(index) {
+    return dbView.getThreadContainingIndex(index)
+                 .getChildHdrAt(0)
+                 .messageKey;
+  };
+
+  let firstThreadId = getThreadId(selectedIndices[0]);
+  let oneThread = true;
+  for (let i = 1; i < selectedIndices.length; i++) {
+    if (getThreadId(selectedIndices[i]) != firstThreadId) {
+      oneThread = false;
+      break;
+    }
+  }
+
+  let selectedMessages = folderDisplay.selectedMessages;
+  if (oneThread)
+    summarizeThread(selectedMessages, aMessageDisplay);
+  else
+    summarizeMultipleSelection(selectedMessages, aMessageDisplay);
+}
 
 /**
- * Given an array of messages which are all in the
- * same thread, summarize them.
+ * Given an array of messages which are all in the same thread, summarize them.
  *
- * @param aSelectedMessages
- *        Array of message headers.
- * @param [aListener]
- *        A listener that implements onLoadStarted and onLoadCompleted.
+ * @param aSelectedMessages Array of message headers.
+ * @param aMessageDisplay   The MessageDisplayWidget object responsible for
+ *                          showing messages.
  */
-function summarizeThread(aSelectedMessages, aListener)
-{
-  if (aSelectedMessages.length == 0)
-    return;
-
+function summarizeThread(aSelectedMessages, aMessageDisplay) {
   try {
-    gSummary = new ThreadSummary(aSelectedMessages, aListener);
+    gSummary = new ThreadSummary(aSelectedMessages, aMessageDisplay);
     gSummary.init();
+    aMessageDisplay.singleMessageDisplay = false;
   } catch (e) {
-    dump("Exception in summarizeThread" + e + "\n");
+    dump("Exception in summarizeThread: " + e + "\n");
     logException(e);
     Components.utils.reportError(e);
     throw(e);
@@ -660,46 +687,40 @@ function summarizeThread(aSelectedMessages, aListener)
 }
 
 /**
- * Given an array of message URIs, cause the message pane
- * to display a summary of them.
+ * Given an array of message URIs, cause the message panel to display a summary
+ * of them.
  *
- * @param aSelectedMessages
- *        Array of message headers.
- * @param [aListener]
- *        A listener that implements onLoadStarted and onLoadCompleted.
+ * @param aSelectedMessages Array of message headers.
+ * @param aMessageDisplay   The MessageDisplayWidget object responsible for
+ *                          showing messages.
  */
-function summarizeMultipleSelection(aSelectedMessages, aListener)
-{
-  if (aSelectedMessages.length == 0)
-    return;
+function summarizeMultipleSelection(aSelectedMessages, aMessageDisplay) {
   try {
-    gSummary = new MultiMessageSummary(aSelectedMessages, aListener);
+    gSummary = new MultiMessageSummary(aSelectedMessages, aMessageDisplay);
     gSummary.init();
+    aMessageDisplay.singleMessageDisplay = false;
   } catch (e) {
-    dump("Exception in summarizeMultipleSelection" + e + "\n");
+    dump("Exception in summarizeMultipleSelection: " + e + "\n");
+    logException(e);
     Components.utils.reportError(e);
     throw(e);
   }
 }
 
-
 /**
- * Helper function to escape some XML chars, so they display properly in
- * innerHTML.
+ * Summarize a message folder; this is mainly a stub function for extensions to
+ * override.  It currently only shows the start page.
  *
- * @param s
- *        input text
- * @return The string with <, >, and & replaced by the corresponding entities.
+ * @param aMessageDisplay The MessageDisplayWidget object responsible for
+ *                        showing messages.
  */
-function escapeXMLchars(s)
-{
-  return s.replace(/[<>&]/g, function(s) {
-      switch (s) {
-          case "<": return "&lt;";
-          case ">": return "&gt;";
-          case "&": return "&amp;";
-          default: throw Error("Unexpected match");
-          }
-      }
-  );
+function summarizeFolder(aMessageDisplay) {
+  aMessageDisplay.clearDisplay();
+
+  // Once in our lifetime is plenty.
+  if (!aMessageDisplay._haveDisplayedStartPage) {
+    loadStartPage(false);
+    aMessageDisplay._haveDisplayedStartPage = true;
+  }
+  aMessageDisplay.singleMessageDisplay = true;
 }
