@@ -80,6 +80,7 @@ nsAbAutoCompleteSearch.prototype = {
   _commentColumn: 0,
   _parser: MailServices.headerParser,
   _abManager: MailServices.ab,
+  applicableHeaders: Set(["addr_to", "addr_cc", "addr_bcc", "addr_reply"]),
 
   // Private methods
 
@@ -290,16 +291,10 @@ nsAbAutoCompleteSearch.prototype = {
   _addToResult: function _addToResult(commentColumn, directory, card,
                                       emailToUse, isPrimaryEmail, result) {
     var emailAddress =
-      this._parser.makeFullAddress(card.displayName,
-                                   card.isMailList ?
-                                   card.getProperty("Notes", "") || card.displayName :
-                                   emailToUse);
-
-    // The old code used to try it manually. I think if the parser can't work
-    // out the address from what we've supplied, then its busted and we're not
-    // going to do any better doing it manually.
-    if (!emailAddress)
-      return;
+      this._parser.makeMailboxObject(card.displayName,
+                                     card.isMailList ?
+                                     card.getProperty("Notes", "") || card.displayName :
+                                     emailToUse).toString();
 
     // If it is a duplicate, then just return and don't add it. The
     // _checkDuplicate function deals with it all for us.
@@ -345,7 +340,13 @@ nsAbAutoCompleteSearch.prototype = {
    */
   startSearch: function startSearch(aSearchString, aSearchParam,
                                     aPreviousResult, aListener) {
+    let params = JSON.parse(aSearchParam);
     var result = new nsAbAutoCompleteResult(aSearchString);
+    if (params.type && !this.applicableHeaders.has(params.type)) {
+      result.searchResult = ACR.RESULT_IGNORED;
+      aListener.onSearchResult(this, result);
+      return;
+    }
 
     // If the search string isn't value, or contains a comma, or the user
     // hasn't enabled autocomplete, then just return no matches / or the
@@ -427,9 +428,8 @@ nsAbAutoCompleteSearch.prototype = {
       // just going to find duplicates.
       while (allABs.hasMoreElements()) {
         var dir = allABs.getNext();
-
         if (dir instanceof Components.interfaces.nsIAbDirectory &&
-            dir.useForAutocomplete(aSearchParam)) {
+            dir.useForAutocomplete(params.idKey)) {
           this._searchCards(searchQuery, dir, result);
           this._searchWithinEmails(emailSearchQuery, fullString, dir, result);
         }
