@@ -1,42 +1,7 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is mozilla.org code.
- *
- * The Initial Developer of the Original Code is
- * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 1999
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Seth Spitzer <sspitzer@netscape.com>
- *   Dan Mosedale <dmose@mozilla.org>
- *   Mark Banner <mark@standard8.demon.co.uk>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either of the GNU General Public License Version 2 or later (the "GPL"),
- * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 /* directory server preferences (used to be dirprefs.c in 4.x) */
 
@@ -51,7 +16,7 @@
 #include "nsIAddrDatabase.h"
 #include "nsAbBaseCID.h"
 #include "nsIAbManager.h"
-#include "nsILocalFile.h"
+#include "nsIFile.h"
 #include "nsWeakReference.h"
 #include "nsIAbMDBDirectory.h"
 #if defined(MOZ_LDAP_XPCOM)
@@ -76,12 +41,12 @@
 static bool dir_IsServerDeleted(DIR_Server * server);
 
 static char *DIR_GetStringPref(const char *prefRoot, const char *prefLeaf, const char *defaultValue);
-static PRInt32 DIR_GetIntPref(const char *prefRoot, const char *prefLeaf, PRInt32 defaultValue);
+static int32_t DIR_GetIntPref(const char *prefRoot, const char *prefLeaf, int32_t defaultValue);
 
 static char * dir_ConvertDescriptionToPrefName(DIR_Server * server);
 
 void DIR_SetFileName(char** filename, const char* leafName);
-static void DIR_SetIntPref(const char *prefRoot, const char *prefLeaf, PRInt32 value, PRInt32 defaultValue);
+static void DIR_SetIntPref(const char *prefRoot, const char *prefLeaf, int32_t value, int32_t defaultValue);
 static DIR_Server *dir_MatchServerPrefToServer(nsVoidArray *wholeList, const char *pref);
 static bool dir_ValidateAndAddNewServer(nsVoidArray *wholeList, const char *fullprefname);
 static void DIR_DeleteServerList(nsVoidArray *wholeList);
@@ -92,9 +57,9 @@ static void DIR_GetPrefsForOneServer(DIR_Server *server);
 static void DIR_InitServer(DIR_Server *server, DirectoryType dirType = (DirectoryType)0);
 static DIR_PrefId  DIR_AtomizePrefName(const char *prefname);
 
-#define DIR_POS_APPEND                     0x80000000
-#define DIR_POS_DELETE                     0x80000001
-static bool DIR_SetServerPosition(nsVoidArray *wholeList, DIR_Server *server, PRInt32 position);
+const int32_t DIR_POS_APPEND = -1;
+const int32_t DIR_POS_DELETE = -2;
+static bool DIR_SetServerPosition(nsVoidArray *wholeList, DIR_Server *server, int32_t position);
 
 /* These two routines should be called to initialize and save 
  * directory preferences from the XP Java Script preferences
@@ -102,8 +67,8 @@ static bool DIR_SetServerPosition(nsVoidArray *wholeList, DIR_Server *server, PR
 static nsresult DIR_GetServerPreferences(nsVoidArray** list);
 static void DIR_SaveServerPreferences(nsVoidArray *wholeList);
 
-static PRInt32 dir_UserId = 0;
-nsVoidArray *dir_ServerList = nsnull;
+static int32_t dir_UserId = 0;
+nsVoidArray *dir_ServerList = nullptr;
 
 /*****************************************************************************
  * Functions for creating the new back end managed DIR_Server list.
@@ -118,11 +83,12 @@ public:
 
 NS_IMPL_ISUPPORTS2(DirPrefObserver, nsISupportsWeakReference, nsIObserver)
 
-NS_IMETHODIMP DirPrefObserver::Observe(nsISupports *aSubject, const char *aTopic, const PRUnichar *aData)
+NS_IMETHODIMP DirPrefObserver::Observe(nsISupports *aSubject, const char *aTopic, const char16_t *aData)
 {
   nsCOMPtr<nsIPrefBranch> prefBranch(do_QueryInterface(aSubject));
-
-  const char *prefname = NS_ConvertUTF16toUTF8(aData).get();
+  nsCString strPrefName;
+  strPrefName.Assign(NS_ConvertUTF16toUTF8(aData));
+  const char *prefname = strPrefName.get();
 
   DIR_PrefId id = DIR_AtomizePrefName(prefname);
 
@@ -146,7 +112,7 @@ NS_IMETHODIMP DirPrefObserver::Observe(nsISupports *aSubject, const char *aTopic
      */
     if (id == idPosition)
     {
-      PRInt32 position;
+      int32_t position;
 
       /* We must not do anything if the new position is the same as the
        * position in the DIR_Server.  This avoids recursion in cases
@@ -163,7 +129,7 @@ NS_IMETHODIMP DirPrefObserver::Observe(nsISupports *aSubject, const char *aTopic
 
     if (id == idDescription)
       // Ensure the local copy of the description is kept up to date.
-      server->description = DIR_GetStringPref(prefname, "description", nsnull);
+      server->description = DIR_GetStringPref(prefname, "description", nullptr);
   }
   /* If the server is not in the unified list, we may need to add it.  Servers
    * are only added when the position, serverName and description are valid.
@@ -177,7 +143,7 @@ NS_IMETHODIMP DirPrefObserver::Observe(nsISupports *aSubject, const char *aTopic
 }
 
 // A pointer to the pref observer
-static DirPrefObserver *prefObserver = nsnull;
+static DirPrefObserver *prefObserver = nullptr;
 
 static nsresult DIR_GetDirServers()
 {
@@ -216,15 +182,15 @@ nsVoidArray* DIR_GetDirectories()
 
 DIR_Server* DIR_GetServerFromList(const char* prefName)
 {
-  DIR_Server* result = nsnull;
+  DIR_Server* result = nullptr;
 
   if (!dir_ServerList)
     DIR_GetDirServers();
 
   if (dir_ServerList)
   {
-    PRInt32 count = dir_ServerList->Count();
-    PRInt32 i;
+    int32_t count = dir_ServerList->Count();
+    int32_t i;
     for (i = 0; i < count; ++i)
     {
       DIR_Server *server = (DIR_Server *)dir_ServerList->ElementAt(i);
@@ -245,7 +211,7 @@ static nsresult SavePrefsFile()
   nsCOMPtr<nsIPrefService> pPref(do_GetService(NS_PREFSERVICE_CONTRACTID, &rv));
   if (NS_FAILED(rv))
     return rv;
-  return pPref->SavePrefFile(nsnull);
+  return pPref->SavePrefFile(nullptr);
 }
 
 nsresult DIR_ShutDown()  /* FEs should call this when the app is shutting down. It frees all DIR_Servers regardless of ref count values! */
@@ -254,7 +220,7 @@ nsresult DIR_ShutDown()  /* FEs should call this when the app is shutting down.
   NS_ENSURE_SUCCESS(rv, rv);
 
   DIR_DeleteServerList(dir_ServerList);
-  dir_ServerList = nsnull;
+  dir_ServerList = nullptr;
 
   /* unregister the preference call back, if necessary.
   * we need to do this as DIR_Shutdown() is called when switching profiles
@@ -273,8 +239,8 @@ nsresult DIR_ContainsServer(DIR_Server* pServer, bool *hasDir)
 {
   if (dir_ServerList)
   {
-    PRInt32 count = dir_ServerList->Count();
-    PRInt32 i;
+    int32_t count = dir_ServerList->Count();
+    int32_t i;
     for (i = 0; i < count; i++)
     {
       DIR_Server* server = (DIR_Server *)(dir_ServerList->ElementAt(i));
@@ -347,7 +313,7 @@ static void DIR_InitServer(DIR_Server *server, DirectoryType dirType)
 
   memset(server, 0, sizeof(DIR_Server));
   server->position = kDefaultPosition;
-  server->uri = nsnull;
+  server->uri = nullptr;
   server->savingServer = false;
   server->dirType = dirType;
 }
@@ -366,13 +332,13 @@ static void DIR_InitServer(DIR_Server *server, DirectoryType dirType)
  *
  * Returns true if the server list was re-sorted.
  */
-static bool DIR_SetServerPosition(nsVoidArray *wholeList, DIR_Server *server, PRInt32 position)
+static bool DIR_SetServerPosition(nsVoidArray *wholeList, DIR_Server *server, int32_t position)
  {
    NS_ENSURE_TRUE(wholeList, false);
 
-   PRInt32    i, count, num;
+   int32_t    i, count, num;
    bool       resort = false;
-   DIR_Server *s=nsnull;
+   DIR_Server *s=nullptr;
    
    switch (position) {
    case DIR_POS_APPEND:
@@ -382,7 +348,7 @@ static bool DIR_SetServerPosition(nsVoidArray *wholeList, DIR_Server *server, PR
      count = wholeList->Count();
      for (i= 0; i < count; i++)
      {
-       if  ((s = (DIR_Server *)wholeList->ElementAt(i)) != nsnull)
+       if  ((s = (DIR_Server *)wholeList->ElementAt(i)) != nullptr)
          if (s == server)
            return false;
      }
@@ -403,7 +369,7 @@ static bool DIR_SetServerPosition(nsVoidArray *wholeList, DIR_Server *server, PR
      
    case DIR_POS_DELETE:
        /* Remove the prefs corresponding to the given server.  If the prefName
-       * value is nsnull, the server has never been saved and there are no
+       * value is nullptr, the server has never been saved and there are no
        * prefs to remove.
      */
      if (server->prefName)
@@ -446,14 +412,14 @@ static bool DIR_SetServerPosition(nsVoidArray *wholeList, DIR_Server *server, PR
      count = wholeList->Count();
      for (i= 0; i < count; i++)
      {
-       if  ((s = (DIR_Server *)wholeList->ElementAt(i)) != nsnull)
+       if  ((s = (DIR_Server *)wholeList->ElementAt(i)) != nullptr)
          if (s == server)
            break;
      }
      
      /* If the server is not in the list, add it to the beginning and re-sort.
      */
-     if (s == nsnull)
+     if (s == nullptr)
      {
        server->position = position;
        wholeList->AppendElement(server);
@@ -492,11 +458,11 @@ static DIR_Server *dir_MatchServerPrefToServer(nsVoidArray *wholeList, const cha
 {
   DIR_Server *server;
 
-  PRInt32 count = wholeList->Count();
-  PRInt32 i;
+  int32_t count = wholeList->Count();
+  int32_t i;
   for (i = 0; i < count; i++)
   {
-    if ((server = (DIR_Server *)wholeList->ElementAt(i)) != nsnull)
+    if ((server = (DIR_Server *)wholeList->ElementAt(i)) != nullptr)
     {
       if (server->prefName && PL_strstr(pref, server->prefName) == pref)
       {
@@ -506,7 +472,7 @@ static DIR_Server *dir_MatchServerPrefToServer(nsVoidArray *wholeList, const cha
       }
     }
   }
-  return nsnull;
+  return nullptr;
 }
 
 /* dir_ValidateAndAddNewServer
@@ -525,18 +491,18 @@ static bool dir_ValidateAndAddNewServer(nsVoidArray *wholeList, const char *full
     char *prefname = (char *)PR_Malloc(endname - fullprefname + 1);
     if (prefname)
     {
-      PRInt32 dirType;
-      char *t1 = nsnull, *t2 = nsnull;
+      int32_t dirType;
+      char *t1 = nullptr, *t2 = nullptr;
 
       PL_strncpyz(prefname, fullprefname, endname - fullprefname + 1);
 
       dirType = DIR_GetIntPref(prefname, "dirType", -1);
       if (dirType != -1 &&
           DIR_GetIntPref(prefname, "position", 0) != 0 &&
-          (t1 = DIR_GetStringPref(prefname, "description", nsnull)) != nsnull)
+          (t1 = DIR_GetStringPref(prefname, "description", nullptr)) != nullptr)
       {
         if (dirType == PABDirectory ||
-           (t2 = DIR_GetStringPref(prefname, "serverName",  nsnull)) != nsnull)
+           (t2 = DIR_GetStringPref(prefname, "serverName",  nullptr)) != nullptr)
         {
           DIR_Server *server = (DIR_Server *)PR_Malloc(sizeof(DIR_Server));
           if (server)
@@ -652,7 +618,7 @@ nsresult DIR_DeleteServerFromList(DIR_Server *server)
     return NS_ERROR_NULL_POINTER;
 
   nsresult rv = NS_OK;
-  nsCOMPtr<nsILocalFile> dbPath;
+  nsCOMPtr<nsIFile> dbPath;
 
   nsCOMPtr<nsIAbManager> abManager = do_GetService(NS_ABMANAGER_CONTRACTID, &rv); 
   if (NS_SUCCEEDED(rv))
@@ -702,15 +668,15 @@ static void DIR_DeleteServerList(nsVoidArray *wholeList)
 {
   if (wholeList)
   {
-    DIR_Server *server = nsnull;
+    DIR_Server *server = nullptr;
   
     /* TBD: Send notifications? */
-    PRInt32 count = wholeList->Count();
-    PRInt32 i;
+    int32_t count = wholeList->Count();
+    int32_t i;
     for (i = count - 1; i >=0; i--)
     {
       server = (DIR_Server *)wholeList->ElementAt(i);
-      if (server != nsnull)
+      if (server != nullptr)
         DIR_DeleteServer(server);
     }
     delete wholeList;
@@ -726,7 +692,7 @@ comparePrefArrayMembers(const void* aElement1, const void* aElement2, void* aDat
 {
     const char* element1 = *static_cast<const char* const *>(aElement1);
     const char* element2 = *static_cast<const char* const *>(aElement2);
-    const PRUint32 offset = *((const PRUint32*)aData);
+    const uint32_t offset = *((const uint32_t*)aData);
 
     // begin the comparison at |offset| chars into the string -
     // this avoids comparing the "ldap_2.servers." portion of every element,
@@ -735,9 +701,9 @@ comparePrefArrayMembers(const void* aElement1, const void* aElement2, void* aDat
 }
 
 static nsresult dir_GetChildList(const nsCString &aBranch,
-                                 PRUint32 *aCount, char ***aChildList)
+                                 uint32_t *aCount, char ***aChildList)
 {
-    PRUint32 branchLen = aBranch.Length();
+    uint32_t branchLen = aBranch.Length();
 
     nsCOMPtr<nsIPrefBranch> prefBranch = do_GetService(NS_PREFSERVICE_CONTRACTID);
     if (!prefBranch) {
@@ -751,7 +717,7 @@ static nsresult dir_GetChildList(const nsCString &aBranch,
 
     // traverse the list, and truncate all the descendant strings to just
     // one branch level below the root branch.
-    for (PRUint32 i = *aCount; i--; ) {
+    for (uint32_t i = *aCount; i--; ) {
         // The prefname we passed to GetChildList was of the form
         // "ldap_2.servers." and we are returned the descendants
         // in the form of "ldap_2.servers.servername.foo"
@@ -773,8 +739,8 @@ static nsresult dir_GetChildList(const nsCString &aBranch,
         // later) entry to test, at any given point. we know we have >= 2
         // elements in the list here, so we just init the two counters sensibly
         // to begin with.
-        PRUint32 cur = 0;
-        for (PRUint32 next = 1; next < *aCount; ++next) {
+        uint32_t cur = 0;
+        for (uint32_t next = 1; next < *aCount; ++next) {
             // check if the elements are equal or unique
             if (!comparePrefArrayMembers(&((*aChildList)[cur]), &((*aChildList)[next]), &branchLen)) {
                 // equal - just free & increment the next element ptr
@@ -800,10 +766,10 @@ static char *DIR_GetStringPref(const char *prefRoot, const char *prefLeaf, const
     nsresult rv;
     nsCOMPtr<nsIPrefBranch> pPref(do_GetService(NS_PREFSERVICE_CONTRACTID, &rv));
     if (NS_FAILED(rv))
-        return nsnull;
+        return nullptr;
 
     nsCString value;
-    nsCAutoString prefLocation(prefRoot);
+    nsAutoCString prefLocation(prefRoot);
 
     prefLocation.Append('.');
     prefLocation.Append(prefLeaf);
@@ -844,9 +810,9 @@ static char *DIR_GetDescription(const char *prefRoot)
   nsCOMPtr<nsIPrefBranch> pPref(do_GetService(NS_PREFSERVICE_CONTRACTID, &rv));
 
   if (NS_FAILED(rv))
-    return nsnull;
+    return nullptr;
 
-  nsCAutoString prefLocation(prefRoot);
+  nsAutoCString prefLocation(prefRoot);
   prefLocation.AppendLiteral(".description");
 
   nsString wvalue;
@@ -856,7 +822,7 @@ static char *DIR_GetDescription(const char *prefRoot)
   if (NS_SUCCEEDED(rv))
     rv = locStr->ToString(getter_Copies(wvalue));
 
-  char *value = nsnull;
+  char *value = nullptr;
   if (!wvalue.IsEmpty())
   {
     value = ToNewCString(NS_ConvertUTF16toUTF8(wvalue));
@@ -877,13 +843,13 @@ static char *DIR_GetDescription(const char *prefRoot)
     // back here.
     rv = pPref->GetCharPref(prefLocation.get(), &value);
     if (NS_FAILED(rv))
-      value = nsnull;
+      value = nullptr;
   }
 
   return value;
 }
 
-static PRInt32 DIR_GetIntPref(const char *prefRoot, const char *prefLeaf, PRInt32 defaultValue)
+static int32_t DIR_GetIntPref(const char *prefRoot, const char *prefLeaf, int32_t defaultValue)
 {
   nsresult rv;
   nsCOMPtr<nsIPrefBranch> pPref(do_GetService(NS_PREFSERVICE_CONTRACTID, &rv));
@@ -891,8 +857,8 @@ static PRInt32 DIR_GetIntPref(const char *prefRoot, const char *prefLeaf, PRInt3
   if (NS_FAILED(rv))
     return defaultValue;
 
-  PRInt32 value;
-  nsCAutoString prefLocation(prefRoot);
+  int32_t value;
+  nsAutoCString prefLocation(prefRoot);
 
   prefLocation.Append('.');
   prefLocation.Append(prefLeaf);
@@ -908,7 +874,7 @@ static PRInt32 DIR_GetIntPref(const char *prefRoot, const char *prefLeaf, PRInt3
 static void DIR_ConvertServerFileName(DIR_Server* pServer)
 {
   char* leafName = pServer->fileName;
-  char* newLeafName = nsnull;
+  char* newLeafName = nullptr;
 #if defined(XP_WIN) || defined(XP_OS2)
   /* jefft -- bug 73349 This is to allow users share same address book.
    * It only works if the user specify a full path filename.
@@ -935,9 +901,9 @@ void DIR_SetFileName(char** fileName, const char* defaultName)
     return;
 
   nsresult rv = NS_OK;
-  nsCOMPtr<nsILocalFile> dbPath;
+  nsCOMPtr<nsIFile> dbPath;
 
-  *fileName = nsnull;
+  *fileName = nullptr;
 
   nsCOMPtr<nsIAbManager> abManager = do_GetService(NS_ABMANAGER_CONTRACTID, &rv); 
   if (NS_SUCCEEDED(rv))
@@ -967,12 +933,12 @@ An extension is not applied
 static char * dir_ConvertDescriptionToPrefName(DIR_Server * server)
 {
 #define MAX_PREF_NAME_SIZE 25
-  char * fileName = nsnull;
+  char * fileName = nullptr;
   char fileNameBuf[MAX_PREF_NAME_SIZE];
-  PRInt32 srcIndex = 0;
-  PRInt32 destIndex = 0;
-  PRInt32 numSrcBytes = 0;
-  const char * descr = nsnull;
+  int32_t srcIndex = 0;
+  int32_t destIndex = 0;
+  int32_t numSrcBytes = 0;
+  const char * descr = nullptr;
   if (server && server->description)
   {
     descr = server->description;
@@ -1000,9 +966,9 @@ static char * dir_ConvertDescriptionToPrefName(DIR_Server * server)
 
 void DIR_SetServerFileName(DIR_Server *server)
 {
-  char * tempName = nsnull; 
-  const char * prefName = nsnull;
-  PRUint32 numHeaderBytes = 0; 
+  char * tempName = nullptr; 
+  const char * prefName = nullptr;
+  uint32_t numHeaderBytes = 0; 
 
   if (server && (!server->fileName || !(*server->fileName)) )
   {
@@ -1050,7 +1016,7 @@ static char *dir_CreateServerPrefName (DIR_Server *server)
      pref name. We'll try to convert the description into a pref name
      and then verify that it is unique. If it is unique then use it... */
   char * leafName = dir_ConvertDescriptionToPrefName(server);
-  char * prefName = nsnull;
+  char * prefName = nullptr;
   bool isUnique = false;
 
   if (!leafName || !*leafName)
@@ -1062,12 +1028,12 @@ static char *dir_CreateServerPrefName (DIR_Server *server)
 
   if (leafName)
   {
-    PRInt32 uniqueIDCnt = 0;
-        char **children = nsnull;
+    int32_t uniqueIDCnt = 0;
+        char **children = nullptr;
     /* we need to verify that this pref string name is unique */
     prefName = PR_smprintf(PREF_LDAP_SERVER_TREE_NAME".%s", leafName);
     isUnique = false;
-    PRUint32 prefCount;
+    uint32_t prefCount;
     nsresult rv = dir_GetChildList(NS_LITERAL_CSTRING(PREF_LDAP_SERVER_TREE_NAME "."),
                                    &prefCount, &children);
     if (NS_SUCCEEDED(rv))
@@ -1075,7 +1041,7 @@ static char *dir_CreateServerPrefName (DIR_Server *server)
       while (!isUnique && prefName)
       {
         isUnique = true; /* now flip the logic and assume we are unique until we find a match */
-        for (PRUint32 i = 0; i < prefCount && isUnique; ++i)
+        for (uint32_t i = 0; i < prefCount && isUnique; ++i)
         {
           if (!PL_strcasecmp(children[i], prefName)) /* are they the same branch? */
             isUnique = false;
@@ -1094,7 +1060,7 @@ static char *dir_CreateServerPrefName (DIR_Server *server)
     if (!isUnique && prefName)
     {
       PR_smprintf_free(prefName);
-      prefName = nsnull;
+      prefName = nullptr;
     }
 
     PR_Free(leafName);
@@ -1158,7 +1124,7 @@ static nsresult dir_GetPrefs(nsVoidArray **list)
         return NS_ERROR_OUT_OF_MEMORY;
 
     char **children;
-    PRUint32 prefCount;
+    uint32_t prefCount;
 
     rv = dir_GetChildList(NS_LITERAL_CSTRING(PREF_LDAP_SERVER_TREE_NAME "."),
                           &prefCount, &children);
@@ -1171,7 +1137,7 @@ static nsresult dir_GetPrefs(nsVoidArray **list)
     if (dir_UserId == 0)
         pPref->GetIntPref(PREF_LDAP_GLOBAL_TREE_NAME".user_id", &dir_UserId);
 
-    for (PRUint32 i = 0; i < prefCount; ++i)
+    for (uint32_t i = 0; i < prefCount; ++i)
     {
         DIR_Server *server;
 
@@ -1234,8 +1200,8 @@ static nsresult DIR_GetServerPreferences(nsVoidArray** list)
   if (NS_FAILED(err))
     return err;
 
-  PRInt32 version = -1;
-  nsVoidArray *newList = nsnull;
+  int32_t version = -1;
+  nsVoidArray *newList = nullptr;
   
   /* Update the ldap list version and see if there are old prefs to migrate. */
   err = pPref->GetIntPref(PREF_LDAP_VERSION_NAME, &version);
@@ -1264,7 +1230,7 @@ static void DIR_SetStringPref(const char *prefRoot, const char *prefLeaf, const
     return;
 
   nsCString defaultPref;
-  nsCAutoString prefLocation(prefRoot);
+  nsAutoCString prefLocation(prefRoot);
 
   prefLocation.Append('.');
   prefLocation.Append(prefLeaf);
@@ -1311,7 +1277,7 @@ static void DIR_SetLocalizedStringPref
   if (NS_FAILED(rv))
     return;
 
-  nsCAutoString prefLocation(prefRoot);
+  nsAutoCString prefLocation(prefRoot);
   prefLocation.Append('.');
 
   nsCOMPtr<nsIPrefBranch> prefBranch;
@@ -1387,15 +1353,15 @@ static void DIR_SetLocalizedStringPref
 }
 
 
-static void DIR_SetIntPref(const char *prefRoot, const char *prefLeaf, PRInt32 value, PRInt32 defaultValue)
+static void DIR_SetIntPref(const char *prefRoot, const char *prefLeaf, int32_t value, int32_t defaultValue)
 {
   nsresult rv;
   nsCOMPtr<nsIPrefBranch> pPref(do_GetService(NS_PREFSERVICE_CONTRACTID, &rv)); 
   if (NS_FAILED(rv)) 
     return;
 
-  PRInt32 defaultPref;
-  nsCAutoString prefLocation(prefRoot);
+  int32_t defaultPref;
+  nsAutoCString prefLocation(prefRoot);
 
   prefLocation.Append('.');
   prefLocation.Append(prefLeaf);
@@ -1407,7 +1373,7 @@ static void DIR_SetIntPref(const char *prefRoot, const char *prefLeaf, PRInt32 v
   }
   else
   {
-    PRInt32 userPref;
+    int32_t userPref;
     if (NS_SUCCEEDED(pPref->GetIntPref(prefLocation.get(), &userPref)))
     {
       if (value != defaultValue)
@@ -1432,7 +1398,7 @@ void DIR_SavePrefsForOneServer(DIR_Server *server)
 
   char *prefstring;
 
-  if (server->prefName == nsnull)
+  if (server->prefName == nullptr)
     server->prefName = dir_CreateServerPrefName(server);
   prefstring = server->prefName;
 
@@ -1463,8 +1429,8 @@ static void DIR_SaveServerPreferences(nsVoidArray *wholeList)
       return;
     }
 
-    PRInt32  i;
-    PRInt32  count = wholeList->Count();
+    int32_t  i;
+    int32_t  count = wholeList->Count();
     DIR_Server *server;
 
     for (i = 0; i < count; i++)
