@@ -1086,7 +1086,6 @@ function onAccountTreeSelect(pageId, account)
 // page has loaded
 function onPanelLoaded(pageId) {
   if (pageId != pendingPageId) {
-
     // if we're reloading the current page, we'll assume the
     // page has asked itself to be completely reloaded from
     // the prefs. to do this, clear out the the old entry in
@@ -1098,7 +1097,6 @@ function onPanelLoaded(pageId) {
       restorePage(currentPageId, currentAccount);
     }
   } else {
-
     restorePage(pendingPageId, pendingAccount);
   }
 
@@ -1109,6 +1107,11 @@ function onPanelLoaded(pageId) {
 
 function pageURL(pageId)
 {
+  // If we have a special non account manager pane (e.g. about:blank),
+  // do not translate it into ChromePackageName URL.
+  if (!pageId.startsWith("am-"))
+    return pageId;
+
   let chromePackageName;
   try {
     // we could compare against "main","server","copies","offline","addressing",
@@ -1480,13 +1483,13 @@ var gAccountTree = {
   _build: function at_build() {
     const Ci = Components.interfaces;
     var bundle = document.getElementById("bundle_prefs");
-    function get(aString) { return bundle.getString(aString); }
-    var panels = [{string: get("prefPanel-server"), src: "am-server.xul"},
-                  {string: get("prefPanel-copies"), src: "am-copies.xul"},
-                  {string: get("prefPanel-synchronization"), src: "am-offline.xul"},
-                  {string: get("prefPanel-diskspace"), src: "am-offline.xul"},
-                  {string: get("prefPanel-addressing"), src: "am-addressing.xul"},
-                  {string: get("prefPanel-junk"), src: "am-junk.xul"}];
+    function getString(aString) { return bundle.getString(aString); }
+    var panels = [{string: getString("prefPanel-server"), src: "am-server.xul"},
+                  {string: getString("prefPanel-copies"), src: "am-copies.xul"},
+                  {string: getString("prefPanel-synchronization"), src: "am-offline.xul"},
+                  {string: getString("prefPanel-diskspace"), src: "am-offline.xul"},
+                  {string: getString("prefPanel-addressing"), src: "am-addressing.xul"},
+                  {string: getString("prefPanel-junk"), src: "am-junk.xul"}];
 
     let accounts = allAccountsSorted(false);
 
@@ -1495,67 +1498,87 @@ var gAccountTree = {
     while (mainTree.firstChild)
       mainTree.removeChild(mainTree.firstChild);
 
-    for each (let account in accounts) {
-      let server = account.incomingServer;
+    for (let account of accounts) {
+      let accountName = null;
+      let accountKey = account.key;
+      let amChrome = "about:blank";
+      let panelsToKeep = [];
 
-      if (server.type == "im" && !Services.prefs.getBoolPref("mail.chat.enabled"))
-        continue;
+      // This "try {} catch {}" block is intentionally very long to catch
+      // unknown exceptions and confine them to this single account.
+      // This may happen from broken accounts. See e.g. bug 813929.
+      // Other accounts can still be shown properly if they are valid.
+      try {
+        let server = account.incomingServer;
 
-      // Create the top level tree-item
+        if (server.type == "im" && !Services.prefs.getBoolPref("mail.chat.enabled"))
+          continue;
+
+        accountName = server.prettyName;
+
+        // Now add our panels.
+        let idents = MailServices.accounts.getIdentitiesForServer(server);
+        if (idents.length) {
+          panelsToKeep.push(panels[0]); // The server panel is valid
+          panelsToKeep.push(panels[1]); // also the copies panel
+          panelsToKeep.push(panels[4]); // and addresssing
+        }
+
+        // Everyone except News, RSS and IM has a junk panel
+        // XXX: unextensible!
+        // The existence of server.spamSettings can't currently be used for this.
+        if (server.type != "nntp" && server.type != "rss" && server.type != "im")
+          panelsToKeep.push(panels[5]);
+
+        // Check offline/diskspace support level.
+        let diskspace = server.supportsDiskSpace;
+        if (server.offlineSupportLevel >= 10 && diskspace)
+          panelsToKeep.push(panels[2]);
+        else if (diskspace)
+          panelsToKeep.push(panels[3]);
+
+        // extensions
+        let catMan = Components.classes["@mozilla.org/categorymanager;1"]
+                               .getService(Ci.nsICategoryManager);
+        const CATEGORY = "mailnews-accountmanager-extensions";
+        let catEnum = catMan.enumerateCategory(CATEGORY);
+        while (catEnum.hasMoreElements()) {
+          var string = Components.interfaces.nsISupportsCString;
+          var entryName = catEnum.getNext().QueryInterface(string).data;
+          var svc = Components.classes[catMan.getCategoryEntry(CATEGORY, entryName)]
+                              .getService(Ci.nsIMsgAccountManagerExtension);
+          if (svc.showPanel(server)) {
+            let bundleName = "chrome://" + svc.chromePackageName +
+                             "/locale/am-" + svc.name + ".properties";
+            let bundle = Services.strings.createBundle(bundleName);
+            let title = bundle.GetStringFromName("prefPanel-" + svc.name);
+            panelsToKeep.push({string: title, src: "am-" + svc.name + ".xul"});
+          }
+        }
+        amChrome = server.accountManagerChrome;
+      } catch(e) {
+        // Show only a placeholder in the account list saying this account
+        // is broken, with no child panels.
+        let accountID = (accountName || accountKey);
+        Components.utils.reportError("Error accessing account " + accountID + ": " + e);
+        accountName = "Invalid account " + accountID;
+        panelsToKeep.length = 0;
+      }
+
+      // Create the top level tree-item.
       var treeitem = document.createElement("treeitem");
       mainTree.appendChild(treeitem);
       var treerow = document.createElement("treerow");
       treeitem.appendChild(treerow);
       var treecell = document.createElement("treecell");
       treerow.appendChild(treecell);
-      treecell.setAttribute("label", server.rootFolder.prettyName);
-
-      // Now add our panels
-      var panelsToKeep = [];
-      let idents = MailServices.accounts.getIdentitiesForServer(server);
-      if (idents.length) {
-        panelsToKeep.push(panels[0]); // The server panel is valid
-        panelsToKeep.push(panels[1]); // also the copies panel
-        panelsToKeep.push(panels[4]); // and addresssing
-      }
-
-      // Everyone except News, RSS and IM has a junk panel
-      // XXX: unextensible!
-      // The existence of server.spamSettings can't currently be used for this.
-      if (server.type != "nntp" && server.type != "rss" && server.type != "im")
-        panelsToKeep.push(panels[5]);
-
-      // Check offline/diskspace support level
-      var offline = server.offlineSupportLevel;
-      var diskspace = server.supportsDiskSpace;
-      if (offline >= 10 && diskspace)
-        panelsToKeep.push(panels[2]);
-      else if (diskspace)
-        panelsToKeep.push(panels[3]);
-
-      // extensions
-      var catMan = Components.classes["@mozilla.org/categorymanager;1"]
-                             .getService(Ci.nsICategoryManager);
-      const CATEGORY = "mailnews-accountmanager-extensions";
-      var catEnum = catMan.enumerateCategory(CATEGORY);
-      while (catEnum.hasMoreElements()) {
-        var string = Components.interfaces.nsISupportsCString;
-        var entryName = catEnum.getNext().QueryInterface(string).data;
-        var svc = Components.classes[catMan.getCategoryEntry(CATEGORY, entryName)]
-                            .getService(Ci.nsIMsgAccountManagerExtension);
-        if (svc.showPanel(server)) {
-          let bundleName = "chrome://" + svc.chromePackageName +
-                           "/locale/am-" + svc.name + ".properties";
-          let bundle = Services.strings.createBundle(bundleName);
-          let title = bundle.GetStringFromName("prefPanel-" + svc.name);
-          panelsToKeep.push({string: title, src: "am-" + svc.name + ".xul"});
-        }
-      }
+      treecell.setAttribute("label", accountName);
+      treeitem.setAttribute("PageTag", amChrome);
 
       if (panelsToKeep.length > 0) {
         var treekids = document.createElement("treechildren");
         treeitem.appendChild(treekids);
-        for each (let panel in panelsToKeep) {
+        for (let panel of panelsToKeep) {
           var kidtreeitem = document.createElement("treeitem");
           treekids.appendChild(kidtreeitem);
           var kidtreerow = document.createElement("treerow");
@@ -1567,28 +1590,26 @@ var gAccountTree = {
           kidtreeitem._account = account;
         }
         treeitem.setAttribute("container", "true");
-        treeitem.id = account.key;
+        treeitem.id = accountKey;
         // Load the 'open' state of the account from localstore.rdf.
-        treeitem.setAttribute("open", this._getAccountOpenState(account.key));
+        treeitem.setAttribute("open", this._getAccountOpenState(accountKey));
         // Let the localstore.rdf automatically save the 'open' state of the
         // account when it is changed.
         treeitem.setAttribute("persist", "open");
       }
-      treeitem.setAttribute("PageTag", server ? server.accountManagerChrome
-                                              : "am-main.xul");
       treeitem._account = account;
     }
 
     markDefaultServer(MailServices.accounts.defaultAccount, null);
 
-    // Now add the outgoing server node
+    // Now add the outgoing server node.
     var treeitem = document.createElement("treeitem");
     mainTree.appendChild(treeitem);
     var treerow = document.createElement("treerow");
     treeitem.appendChild(treerow);
     var treecell = document.createElement("treecell");
     treerow.appendChild(treecell);
-    treecell.setAttribute("label", bundle.getString("prefPanel-smtp"));
+    treecell.setAttribute("label", getString("prefPanel-smtp"));
     treeitem.setAttribute("PageTag", "am-smtp.xul");
   }
 };
