@@ -26,6 +26,7 @@
 #include "nsMsgMessageFlags.h"
 #include "nsISupportsArray.h"
 #include "nsAlgorithm.h"
+#include <algorithm>
 
 // This stuff lives in the base class because the IMAP search syntax
 // is used by the Dredd SEARCH command as well as IMAP itself
@@ -121,8 +122,8 @@ NS_IMETHODIMP nsMsgSearchAdapter::SendUrl ()
   return NS_OK;
 }
 
-/* void CurrentUrlDone (in long exitCode); */
-NS_IMETHODIMP nsMsgSearchAdapter::CurrentUrlDone(int32_t exitCode)
+/* void CurrentUrlDone (in nsresult exitCode); */
+NS_IMETHODIMP nsMsgSearchAdapter::CurrentUrlDone(nsresult exitCode)
 {
   // base implementation doesn't need to do anything.
   return NS_OK;
@@ -148,12 +149,12 @@ NS_IMETHODIMP nsMsgSearchAdapter::AddHit(nsMsgKey key)
 
 
 char *
-nsMsgSearchAdapter::GetImapCharsetParam(const PRUnichar *destCharset)
+nsMsgSearchAdapter::GetImapCharsetParam(const char16_t *destCharset)
 {
   char *result = nullptr;
 
   // Specify a character set unless we happen to be US-ASCII.
-  if (NS_strcmp(destCharset, NS_LITERAL_STRING("us-ascii").get()))
+  if (NS_strcmp(destCharset, MOZ_UTF16("us-ascii")))
       result = PR_smprintf("%s%s", nsMsgSearchAdapter::m_kImapCharset, NS_ConvertUTF16toUTF8(destCharset).get());
 
   return result;
@@ -161,35 +162,35 @@ nsMsgSearchAdapter::GetImapCharsetParam(const PRUnichar *destCharset)
 
 /*
    09/21/2000 - taka@netscape.com
-   This method is bogus. Escape must be done against char * not PRUnichar *
+   This method is bogus. Escape must be done against char * not char16_t *
    should be rewritten later.
    for now, just duplicate the string.
 */
-PRUnichar *nsMsgSearchAdapter::EscapeSearchUrl (const PRUnichar *nntpCommand)
+char16_t *nsMsgSearchAdapter::EscapeSearchUrl (const char16_t *nntpCommand)
 {
   return nntpCommand ? NS_strdup(nntpCommand) : nullptr;
 }
 
 /*
    09/21/2000 - taka@netscape.com
-   This method is bogus. Escape must be done against char * not PRUnichar *
+   This method is bogus. Escape must be done against char * not char16_t *
    should be rewritten later.
    for now, just duplicate the string.
 */
-PRUnichar *
-nsMsgSearchAdapter::EscapeImapSearchProtocol(const PRUnichar *imapCommand)
+char16_t *
+nsMsgSearchAdapter::EscapeImapSearchProtocol(const char16_t *imapCommand)
 {
   return imapCommand ? NS_strdup(imapCommand) : nullptr;
 }
 
 /*
    09/21/2000 - taka@netscape.com
-   This method is bogus. Escape must be done against char * not PRUnichar *
+   This method is bogus. Escape must be done against char * not char16_t *
    should be rewritten later.
    for now, just duplicate the string.
 */
-PRUnichar *
-nsMsgSearchAdapter::EscapeQuoteImapSearchProtocol(const PRUnichar *imapCommand)
+char16_t *
+nsMsgSearchAdapter::EscapeQuoteImapSearchProtocol(const char16_t *imapCommand)
 {
   return imapCommand ? NS_strdup(imapCommand) : nullptr;
 }
@@ -245,7 +246,8 @@ nsMsgSearchAdapter::GetSearchCharsets(nsAString &srcCharset, nsAString &dstChars
     }
   }
   srcCharset = m_defaultCharset.IsEmpty() ?
-    NS_LITERAL_STRING("ISO-8859-1") : m_defaultCharset;
+    static_cast<const nsAString&>(NS_LITERAL_STRING("ISO-8859-1")) :
+    m_defaultCharset;
 
   if (m_scope)
   {
@@ -290,7 +292,7 @@ nsMsgSearchAdapter::GetSearchCharsets(nsAString &srcCharset, nsAString &dstChars
   return NS_OK;
 }
 
-nsresult nsMsgSearchAdapter::EncodeImapTerm (nsIMsgSearchTerm *term, bool reallyDredd, const PRUnichar *srcCharset, const PRUnichar *destCharset, char **ppOutTerm)
+nsresult nsMsgSearchAdapter::EncodeImapTerm (nsIMsgSearchTerm *term, bool reallyDredd, const char16_t *srcCharset, const char16_t *destCharset, char **ppOutTerm)
 {
   NS_ENSURE_ARG_POINTER(term);
   NS_ENSURE_ARG_POINTER(ppOutTerm);
@@ -523,7 +525,7 @@ nsresult nsMsgSearchAdapter::EncodeImapTerm (nsIMsgSearchTerm *term, bool really
 
       if (IS_STRING_ATTRIBUTE(attrib))
       {
-        PRUnichar *convertedValue; // = reallyDredd ? MSG_EscapeSearchUrl (term->m_value.u.string) : msg_EscapeImapSearchProtocol(term->m_value.u.string);
+        char16_t *convertedValue; // = reallyDredd ? MSG_EscapeSearchUrl (term->m_value.u.string) : msg_EscapeImapSearchProtocol(term->m_value.u.string);
         nsString searchTermValue;
         searchValue->GetStr(searchTermValue);
         // Ugly switch for Korean mail/news charsets.
@@ -540,7 +542,7 @@ nsresult nsMsgSearchAdapter::EncodeImapTerm (nsIMsgSearchTerm *term, bool really
         convertedValue = reallyDredd ? EscapeSearchUrl (searchTermValue.get()) :
         EscapeImapSearchProtocol(searchTermValue.get());
         useQuotes = ((!reallyDredd ||
-                    (nsDependentString(convertedValue).FindChar(PRUnichar(' ')) != -1)) &&
+                    (nsDependentString(convertedValue).FindChar(char16_t(' ')) != -1)) &&
            (attrib != nsMsgSearchAttrib::Keywords));
         // now convert to char* and escape quoted_specials
         nsAutoCString valueStr;
@@ -656,7 +658,7 @@ nsresult nsMsgSearchAdapter::EncodeImapValue(char *encoding, const char *value,
 }
 
 
-nsresult nsMsgSearchAdapter::EncodeImap (char **ppOutEncoding, nsISupportsArray *searchTerms, const PRUnichar *srcCharset, const PRUnichar *destCharset, bool reallyDredd)
+nsresult nsMsgSearchAdapter::EncodeImap (char **ppOutEncoding, nsISupportsArray *searchTerms, const char16_t *srcCharset, const char16_t *destCharset, bool reallyDredd)
 {
   // i've left the old code (before using CBoolExpression for debugging purposes to make sure that
   // the new code generates the same encoding string as the old code.....
@@ -898,7 +900,7 @@ nsMsgSearchValidityTable::GetAvailableOperators(nsMsgSearchAttribValue aAttribut
     if (aAttribute == nsMsgSearchAttrib::Default)
       attr = m_defaultAttrib;
     else
-      attr = NS_MIN(aAttribute,
+      attr = std::min(aAttribute,
                     (nsMsgSearchAttribValue)nsMsgSearchAttrib::OtherHeader);
 
     uint32_t totalOperators=0;
@@ -1166,7 +1168,7 @@ nsMsgSearchValidityManager::SetOtherHeadersInTable (nsIMsgSearchValidityTable *a
 
   NS_ASSERTION(nsMsgSearchAttrib::OtherHeader + numHeaders < nsMsgSearchAttrib::kNumMsgSearchAttributes, "more headers than the table can hold");
 
-  uint32_t maxHdrs = NS_MIN(nsMsgSearchAttrib::OtherHeader + numHeaders + 1,
+  uint32_t maxHdrs = std::min(nsMsgSearchAttrib::OtherHeader + numHeaders + 1,
                             (uint32_t)nsMsgSearchAttrib::kNumMsgSearchAttributes);
   for (uint32_t i=nsMsgSearchAttrib::OtherHeader+1;i< maxHdrs;i++)
   {
