@@ -159,7 +159,6 @@ nsresult nsMsgComposeService::Init()
   if (pbi)
     rv = pbi->AddObserver(PREF_MAIL_COMPOSE_MAXRECYCLEDWINDOWS, this, true);
 
-  mOpenComposeWindows.Init();
   Reset();
 
   AddGlobalHtmlDomains();
@@ -326,7 +325,7 @@ void nsMsgComposeService::CloseHiddenCachedWindow(nsIDOMWindow *domWindow)
 }
 
 NS_IMETHODIMP
-nsMsgComposeService::Observe(nsISupports *aSubject, const char *aTopic, const PRUnichar *someData)
+nsMsgComposeService::Observe(nsISupports *aSubject, const char *aTopic, const char16_t *someData)
 {
   if (!strcmp(aTopic, "profile-do-change") || !strcmp(aTopic, "quit-application"))
   {
@@ -414,12 +413,9 @@ nsMsgComposeService::GetOrigWindowSelection(MSG_ComposeType type, nsIMsgWindow *
   rv = aMsgWindow->GetRootDocShell(getter_AddRefs(rootDocShell));
   NS_ENSURE_SUCCESS(rv, rv);
 
-  nsCOMPtr<nsIDocShellTreeNode> rootDocShellAsNode(do_QueryInterface(rootDocShell, &rv));
-  NS_ENSURE_SUCCESS(rv, rv);
-
   nsCOMPtr<nsIDocShellTreeItem> childAsItem;
-  rv = rootDocShellAsNode->FindChildWithName(NS_LITERAL_STRING("messagepane").get(),
-                                             true, false, nullptr, nullptr, getter_AddRefs(childAsItem));
+  rv = rootDocShell->FindChildWithName(MOZ_UTF16("messagepane"),
+                                       true, false, nullptr, nullptr, getter_AddRefs(childAsItem));
   NS_ENSURE_SUCCESS(rv, rv);
 
   nsCOMPtr<nsIDocShell> docShell(do_QueryInterface(childAsItem, &rv));
@@ -445,12 +441,15 @@ nsMsgComposeService::GetOrigWindowSelection(MSG_ComposeType type, nsIMsgWindow *
     // two words selected in order to quote just the selected text
     if (requireMultipleWords)
     {
+      if (selPlain.IsEmpty())
+        return NS_ERROR_ABORT;
+
       nsCOMPtr<nsILineBreaker> lineBreaker = do_GetService(NS_LBRK_CONTRACTID, &rv);
 
       if (NS_SUCCEEDED(rv))
       {
         const uint32_t length = selPlain.Length();
-        const PRUnichar* unicodeStr = selPlain.get();
+        const char16_t* unicodeStr = selPlain.get();
         int32_t endWordPos = lineBreaker->Next(unicodeStr, length, 0);
         
         // If there's not even one word, then there's not multiple words
@@ -458,7 +457,7 @@ nsMsgComposeService::GetOrigWindowSelection(MSG_ComposeType type, nsIMsgWindow *
           return NS_ERROR_ABORT;
 
         // If after the first word is only space, then there's not multiple words
-        const PRUnichar* end;
+        const char16_t* end;
         for (end = unicodeStr + endWordPos; NS_IsSpace(*end); end++)
           ;
         if (!*end)
@@ -574,7 +573,7 @@ nsMsgComposeService::OpenComposeWindow(const char *msgComposeWindowURL, nsIMsgDB
                             nsINetUtil::ESCAPE_URL_FILE_BASENAME | nsINetUtil::ESCAPE_URL_FORCED,
                             unescapedName);
           pMsgCompFields->SetNewsgroups(NS_ConvertUTF8toUTF16(unescapedName));
-          pMsgCompFields->SetNewshost(host.get());
+          pMsgCompFields->SetNewspostUrl(host.get());
         }
         else
         {
@@ -824,7 +823,11 @@ nsMsgComposeService::CacheWindow(nsIDOMWindow *aWindow, bool aComposeHTML, nsIMs
   nsCOMPtr<nsIDocShellTreeItem> treeItem(do_QueryInterface(window->GetDocShell(), &rv));
   NS_ENSURE_SUCCESS(rv, rv);
 
-  nsCOMPtr<nsIXULWindow> xulWindow(do_GetInterface(treeItem, &rv));
+  nsCOMPtr <nsIDocShellTreeOwner> treeOwner;
+  rv = treeItem->GetTreeOwner(getter_AddRefs(treeOwner));
+  NS_ENSURE_SUCCESS(rv,rv);
+
+  nsCOMPtr<nsIXULWindow> xulWindow(do_GetInterface(treeOwner, &rv));
   NS_ENSURE_SUCCESS(rv, rv);
 
   int32_t i;
