@@ -462,14 +462,14 @@ void nsPop3Service::AlertServerBusy(nsIMsgMailNewsUrl *url)
   rv = server->GetPrettyName(accountName);
   NS_ENSURE_SUCCESS_VOID(rv);
 
-  const PRUnichar *params[] = { accountName.get() };
+  const char16_t *params[] = { accountName.get() };
   nsString alertString;
   nsString dialogTitle;
   bundle->FormatStringFromName(
-    NS_LITERAL_STRING("pop3ServerBusy").get(),
+    MOZ_UTF16("pop3ServerBusy"),
     params, 1, getter_Copies(alertString));
   bundle->FormatStringFromName(
-    NS_LITERAL_STRING("pop3ErrorDialogTitle").get(),
+    MOZ_UTF16("pop3ErrorDialogTitle"),
     params, 1, getter_Copies(dialogTitle));
   if (!alertString.IsEmpty())
     dialog->Alert(dialogTitle.get(), alertString.get());
