@@ -705,9 +705,9 @@ nsresult nsPop3Sink::HandleTempDownloadFailed(nsIMsgWindow *msgWindow)
 
   m_newMailParser->m_newMsgHdr->GetMime2DecodedSubject(subjectStr);
   m_newMailParser->m_newMsgHdr->GetMime2DecodedAuthor(fromStr);
-  const PRUnichar *params[] = { fromStr.get(), subjectStr.get() };
+  const char16_t *params[] = { fromStr.get(), subjectStr.get() };
   bundle->FormatStringFromName(
-    NS_LITERAL_STRING("pop3TmpDownloadError").get(),
+    MOZ_UTF16("pop3TmpDownloadError"),
     params, 2, getter_Copies(confirmString));
   nsCOMPtr<nsIDOMWindow> parentWindow;
   nsCOMPtr<nsIPromptService> promptService = do_GetService(NS_PROMPTSERVICE_CONTRACTID);
