@@ -12,10 +12,10 @@ import sys
 SCRIPT_DIR = os.path.abspath(os.path.realpath(os.path.dirname(__file__)))
 sys.path.insert(0, SCRIPT_DIR);
 
+import glob
 import json
 import mozcrash
 import mozinfo
-import mozlog
 import mozprocess
 import mozrunner
 import optparse
@@ -27,8 +27,9 @@ import tempfile
 import time
 import traceback
 import urllib2
+import zipfile
 
-from automationutils import environment, getDebuggerInfo, isURL, KeyValueParseError, parseKeyValue, processLeakLog, systemMemory, dumpScreen
+from automationutils import environment, getDebuggerInfo, isURL, KeyValueParseError, parseKeyValue, processLeakLog, systemMemory, dumpScreen, ShutdownLeaks
 from datetime import datetime
 from manifestparser import TestManifest
 from mochitest_options import MochitestOptions
@@ -51,6 +52,16 @@ def resetGlobalLog():
   log.addHandler(handler)
 resetGlobalLog()
 
+###########################
+# Option for NSPR logging #
+###########################
+
+# Set the desired log modules you want an NSPR log be produced by a try run for, or leave blank to disable the feature.
+# This will be passed to NSPR_LOG_MODULES environment variable. Try run will then put a download link for the log file
+# on tbpl.mozilla.org.
+
+NSPR_LOG_MODULES = ""
+
 ####################
 # PROCESS HANDLING #
 ####################
@@ -288,6 +299,8 @@ class MochitestUtilsMixin(object):
         logFile -- logs test run to an absolute path
         totalChunks -- how many chunks to split tests into
         thisChunk -- which chunk to run
+        startAt -- name of test to start at
+        endAt -- name of test to end at
         timeout -- per-test timeout in seconds
         repeat -- How many times to repeat the test, ie: repeat=1 will run the test twice.
     """
@@ -314,6 +327,10 @@ class MochitestUtilsMixin(object):
         self.urlOpts.append("thisChunk=%d" % options.thisChunk)
       if options.chunkByDir:
         self.urlOpts.append("chunkByDir=%d" % options.chunkByDir)
+      if options.startAt:
+        self.urlOpts.append("startAt=%s" % options.startAt)
+      if options.endAt:
+        self.urlOpts.append("endAt=%s" % options.endAt)
       if options.shuffle:
         self.urlOpts.append("shuffle=1")
       if "MOZ_HIDE_RESULTS_TABLE" in env and env["MOZ_HIDE_RESULTS_TABLE"] == "1":
@@ -336,6 +353,14 @@ class MochitestUtilsMixin(object):
         self.urlOpts.append("failureFile=%s" % self.getFullPath(options.failureFile))
       if options.runSlower:
         self.urlOpts.append("runSlower=true")
+      if options.debugOnFailure:
+        self.urlOpts.append("debugOnFailure=true")
+      if options.dumpOutputDirectory:
+        self.urlOpts.append("dumpOutputDirectory=%s" % encodeURIComponent(options.dumpOutputDirectory))
+      if options.dumpAboutMemoryAfterTest:
+        self.urlOpts.append("dumpAboutMemoryAfterTest=true")
+      if options.dumpDMDAfterTest:
+        self.urlOpts.append("dumpDMDAfterTest=true")
 
   def buildTestPath(self, options):
     """ Build the url path to the specific test harness and test file or directory
@@ -345,16 +370,22 @@ class MochitestUtilsMixin(object):
       manifest = TestManifest(strict=False)
       manifest.read(options.manifestFile)
       # Bug 883858 - return all tests including disabled tests
-      tests = manifest.active_tests(disabled=False, **mozinfo.info)
+      tests = manifest.active_tests(disabled=True, **mozinfo.info)
+      # We need to ensure we match on a complete directory name matching the
+      # test root, and not a substring somewhere else in the path.
+      test_root = os.path.sep + self.getTestRoot(options) + os.path.sep
       paths = []
       for test in tests:
-        tp = test['path'].split(self.getTestRoot(options), 1)[1].strip('/')
+        tp = test['path'].split(test_root, 1)[1].replace('\\', '/').strip('/')
 
         # Filter out tests if we are using --test-path
         if options.testPath and not tp.startswith(options.testPath):
           continue
 
-        paths.append({'path': tp})
+        testob = {'path': tp}
+        if test.has_key('disabled'):
+          testob['disabled'] = test['disabled']
+        paths.append(testob)
 
       # Bug 883865 - add this functionality into manifestDestiny
       with open('tests.json', 'w') as manifestFile:
@@ -550,6 +581,7 @@ class Mochitest(MochitestUtilsMixin):
     """ create the profile and add optional chrome bits and files if requested """
     if options.browserChrome and options.timeout:
       options.extraPrefs.append("testing.browserTestHarness.timeout=%d" % options.timeout)
+    options.extraPrefs.append("browser.tabs.remote=%s" % ('true' if options.e10s else 'false'))
 
     # get extensions to install
     extensions = self.getExtensionsToInstall(options)
@@ -607,9 +639,10 @@ class Mochitest(MochitestUtilsMixin):
     self.copyExtraFilesToProfile(options)
     return manifest
 
-  def buildBrowserEnv(self, options):
+  def buildBrowserEnv(self, options, debugger=False):
     """build the environment variables for the specific test and operating system"""
-    browserEnv = self.environment(xrePath=options.xrePath)
+    browserEnv = self.environment(xrePath=options.xrePath, debugger=debugger,
+                                  dmdPath=options.dmdPath)
 
     # These variables are necessary for correct application startup; change
     # via the commandline at your own risk.
@@ -627,6 +660,15 @@ class Mochitest(MochitestUtilsMixin):
     if options.fatalAssertions:
       browserEnv["XPCOM_DEBUG_BREAK"] = "stack-and-abort"
 
+    # Produce an NSPR log, is setup (see NSPR_LOG_MODULES global at the top of
+    # this script).
+    self.nsprLogs = NSPR_LOG_MODULES and "MOZ_UPLOAD_DIR" in os.environ
+    if self.nsprLogs:
+      browserEnv["NSPR_LOG_MODULES"] = NSPR_LOG_MODULES
+
+      browserEnv["NSPR_LOG_FILE"] = "%s/nspr.log" % tempfile.gettempdir()
+      browserEnv["GECKO_SEPARATE_NSPR_LOGS"] = "1"
+
     return browserEnv
 
   def cleanup(self, manifest, options):
@@ -738,7 +780,9 @@ class Mochitest(MochitestUtilsMixin):
              debuggerInfo=None,
              symbolsPath=None,
              timeout=-1,
-             onLaunch=None):
+             onLaunch=None,
+             webapprtChrome=False,
+             hide_subtests=False):
     """
     Run the app, log the duration it took to execute, return the status code.
     Kills the app if it runs for longer than |maxTime| seconds, or outputs nothing for |timeout| seconds.
@@ -812,17 +856,31 @@ class Mochitest(MochitestUtilsMixin):
     # https://bugzilla.mozilla.org/show_bug.cgi?id=916512
     args.append('-foreground')
     if testUrl:
-       args.append(testUrl)
+      if debuggerInfo and debuggerInfo['requiresEscapedArgs']:
+        testUrl = testUrl.replace("&", "\\&")
+      args.append(testUrl)
+
+    if mozinfo.info["debug"] and not webapprtChrome:
+      shutdownLeaks = ShutdownLeaks(log.info)
+    else:
+      shutdownLeaks = None
 
     # create an instance to process the output
     outputHandler = self.OutputHandler(harness=self,
                                        utilityPath=utilityPath,
                                        symbolsPath=symbolsPath,
                                        dump_screen_on_timeout=not debuggerInfo,
+                                       hide_subtests=hide_subtests,
+                                       shutdownLeaks=shutdownLeaks,
       )
 
-    # if the output handler is a pipe, it will process output via the subprocess
-    kp_kwargs = {} if outputHandler.pipe else {'processOutputLine': [outputHandler]}
+    def timeoutHandler():
+      outputHandler.log_output_buffer()
+      browserProcessId = outputHandler.browserProcessId
+      self.handleTimeout(timeout, proc, utilityPath, debuggerInfo, browserProcessId)
+    kp_kwargs = {'kill_on_timeout': False,
+                 'onTimeout': [timeoutHandler]}
+    kp_kwargs['processOutputLine'] = [outputHandler]
 
     # create mozrunner instance and start the system under test process
     self.lastTestSeen = self.test_name
@@ -853,9 +911,6 @@ class Mochitest(MochitestUtilsMixin):
     proc = runner.process_handler
     log.info("INFO | runtests.py | Application pid: %d", proc.pid)
 
-    # set process information on the output handler
-    outputHandler.setProcess(proc if interactive else proc.proc, timeout)
-
     if onLaunch is not None:
       # Allow callers to specify an onLaunch callback to be fired after the
       # app is launched.
@@ -879,11 +934,6 @@ class Mochitest(MochitestUtilsMixin):
     # finalize output handler
     outputHandler.finish(didTimeout)
 
-    # handle timeout
-    if didTimeout:
-      browserProcessId = outputHandler.browserProcessId
-      self.handleTimeout(timeout, proc, utilityPath, debuggerInfo, browserProcessId)
-
     # record post-test information
     if status:
       log.info("TEST-UNEXPECTED-FAIL | %s | application terminated with exit code %s", self.lastTestSeen, status)
@@ -928,7 +978,7 @@ class Mochitest(MochitestUtilsMixin):
 
     self.leak_report_file = os.path.join(options.profilePath, "runtests_leaks.log")
 
-    browserEnv = self.buildBrowserEnv(options)
+    browserEnv = self.buildBrowserEnv(options, debuggerInfo is not None)
     if browserEnv is None:
       return 1
 
@@ -993,7 +1043,9 @@ class Mochitest(MochitestUtilsMixin):
                            debuggerInfo=debuggerInfo,
                            symbolsPath=options.symbolsPath,
                            timeout=timeout,
-                           onLaunch=onLaunch
+                           onLaunch=onLaunch,
+                           webapprtChrome=options.webapprtChrome,
+                           hide_subtests=options.hide_subtests
                            )
     except KeyboardInterrupt:
       log.info("runtests.py | Received keyboard interrupt.\n");
@@ -1010,6 +1062,12 @@ class Mochitest(MochitestUtilsMixin):
     self.stopWebSocketServer(options)
     processLeakLog(self.leak_report_file, options.leakThreshold)
 
+    if self.nsprLogs:
+      with zipfile.ZipFile("%s/nsprlog.zip" % browserEnv["MOZ_UPLOAD_DIR"], "w", zipfile.ZIP_DEFLATED) as logzip:
+        for logfile in glob.glob("%s/nspr*.log*" % tempfile.gettempdir()):
+          logzip.write(logfile)
+          os.remove(logfile)
+
     log.info("runtests.py | Running tests: end.")
 
     if manifest is not None:
@@ -1027,15 +1085,20 @@ class Mochitest(MochitestUtilsMixin):
 
   class OutputHandler(object):
     """line output handler for mozrunner"""
-    def __init__(self, harness, utilityPath, symbolsPath=None, dump_screen_on_timeout=True):
+    def __init__(self, harness, utilityPath, symbolsPath=None, dump_screen_on_timeout=True,
+                 hide_subtests=False, shutdownLeaks=None):
       """
       harness -- harness instance
       dump_screen_on_timeout -- whether to dump the screen on timeout
       """
       self.harness = harness
+      self.output_buffer = []
+      self.running_test = False
       self.utilityPath = utilityPath
       self.symbolsPath = symbolsPath
       self.dump_screen_on_timeout = dump_screen_on_timeout
+      self.hide_subtests = hide_subtests
+      self.shutdownLeaks = shutdownLeaks
 
       # perl binary to use
       self.perl = which('perl')
@@ -1046,11 +1109,7 @@ class Mochitest(MochitestUtilsMixin):
       self.browserProcessId = None
 
       # stack fixer function and/or process
-      self.stackFixerFunction, self.stackFixerCommand = self.stackFixer()
-      self.stackFixerProcess = None
-
-      # if there is a stack fixer subprocess, function as a pipe
-      self.pipe = bool(self.stackFixerCommand)
+      self.stackFixerFunction, self.stackFixerProcess = self.stackFixer()
 
     def processOutputLine(self, line):
       """per line handler of output for mozprocess"""
@@ -1062,10 +1121,12 @@ class Mochitest(MochitestUtilsMixin):
       """returns ordered list of output handlers"""
       return [self.fix_stack,
               self.format,
-              self.record_last_test,
               self.dumpScreenOnTimeout,
               self.metro_subprocess_id,
+              self.trackShutdownLeaks,
+              self.check_test_failure,
               self.log,
+              self.record_last_test,
               ]
 
     def stackFixer(self):
@@ -1077,7 +1138,7 @@ class Mochitest(MochitestUtilsMixin):
       if not mozinfo.info.get('debug'):
         return None, None
 
-      stackFixerFunction = stackFixerCommand = None
+      stackFixerFunction = stackFixerProcess = None
 
       def import_stackFixerModule(module_name):
         sys.path.insert(0, self.utilityPath)
@@ -1094,25 +1155,31 @@ class Mochitest(MochitestUtilsMixin):
       elif mozinfo.isLinux and self.perl:
         # Run logsource through fix-linux-stack.pl (uses addr2line)
         # This method is preferred for developer machines, so we don't have to run "make buildsymbols".
-
         stackFixerCommand = [self.perl, os.path.join(self.utilityPath, "fix-linux-stack.pl")]
+        stackFixerProcess = subprocess.Popen(stackFixerCommand, stdin=subprocess.PIPE,
+                                             stdout=subprocess.PIPE)
+        def fixFunc(line):
+          stackFixerProcess.stdin.write(line + '\n')
+          return stackFixerProcess.stdout.readline().strip()
 
-      return (stackFixerFunction, stackFixerCommand)
+        stackFixerFunction = fixFunc
 
-    def setProcess(self, proc, outputTimeout=None):
-      if self.stackFixerCommand:
-        self.stackFixerProcess = mozprocess.ProcessHandler(self.stackFixerCommand,
-                                                           stdin=proc.stdout,
-                                                           processOutputLine=[self],
-          )
-        self.stackFixerProcess.run(outputTimeout=outputTimeout)
+      return (stackFixerFunction, stackFixerProcess)
 
     def finish(self, didTimeout):
       if self.stackFixerProcess:
-        status = self.stackFixerProcess.wait()
+        self.stackFixerProcess.communicate()
+        status = self.stackFixerProcess.returncode
         if status and not didTimeout:
           log.info("TEST-UNEXPECTED-FAIL | runtests.py | Stack fixer process exited with code %d during test run", status)
 
+      if self.shutdownLeaks:
+        self.shutdownLeaks.process()
+
+    def log_output_buffer(self):
+        if self.output_buffer:
+            lines = ['  %s' % line for line in self.output_buffer]
+            log.info("Buffered test output:\n%s" % '\n'.join(lines))
 
     # output line handlers:
     # these take a line and return a line
@@ -1126,14 +1193,9 @@ class Mochitest(MochitestUtilsMixin):
       """format the line"""
       return line.rstrip().decode("UTF-8", "ignore")
 
-    def record_last_test(self, line):
-      """record last test on harness"""
-      if "TEST-START" in line and "|" in line:
-        self.harness.lastTestSeen = line.split("|")[1].strip()
-      return line
-
     def dumpScreenOnTimeout(self, line):
       if self.dump_screen_on_timeout and "TEST-UNEXPECTED-FAIL" in line and "Test timed out" in line:
+        self.log_output_buffer()
         self.harness.dumpScreen(self.utilityPath)
       return line
 
@@ -1146,8 +1208,36 @@ class Mochitest(MochitestUtilsMixin):
           log.info("INFO | runtests.py | metro browser sub process id detected: %s", self.browserProcessId)
       return line
 
+    def trackShutdownLeaks(self, line):
+      if self.shutdownLeaks:
+        self.shutdownLeaks.log(line)
+      return line
+
+    def check_test_failure(self, line):
+      if 'TEST-END' in line:
+        self.running_test = False
+        if any('TEST-UNEXPECTED' in l for l in self.output_buffer):
+          self.log_output_buffer()
+      return line
+
     def log(self, line):
-      log.info(line)
+      if self.hide_subtests and self.running_test:
+        self.output_buffer.append(line)
+      else:
+        # hack to make separators align nicely, remove when we use mozlog
+        if self.hide_subtests and 'TEST-END' in line:
+            index = line.index('TEST-END') + len('TEST-END')
+            line = line[:index] + ' ' * (len('TEST-START')-len('TEST-END')) + line[index:]
+        log.info(line)
+      return line
+
+    def record_last_test(self, line):
+      """record last test on harness"""
+      if "TEST-START" in line and "|" in line:
+        if not line.endswith('Shutdown'):
+          self.output_buffer = []
+          self.running_test = True
+        self.harness.lastTestSeen = line.split("|")[1].strip()
       return line
 
 
