@@ -2,6 +2,8 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
+Components.utils.import("resource://gre/modules/Services.jsm");
+
 var gStateObject;
 var gTreeData;
 
@@ -107,9 +109,7 @@ function restoreSession() {
 }
 
 function startNewSession() {
-  if (Components.classes["@mozilla.org/preferences-service;1"]
-                .getService(Components.interfaces.nsIPrefBranch)
-                .getIntPref("browser.startup.page") == 1)
+  if (Services.prefs.getIntPref("browser.startup.page") == 1)
     getBrowserWindow().BrowserHome();
   else
     getBrowserWindow().getBrowser().loadURI("about:blank");
@@ -144,14 +144,6 @@ function onListKeyDown(aEvent) {
     if (aEvent.ctrlKey && !treeView.isContainer(ix))
       restoreSingleTab(ix, aEvent.shiftKey);
     break;
-  case KeyEvent.DOM_VK_UP:
-  case KeyEvent.DOM_VK_DOWN:
-  case KeyEvent.DOM_VK_PAGE_UP:
-  case KeyEvent.DOM_VK_PAGE_DOWN:
-  case KeyEvent.DOM_VK_HOME:
-  case KeyEvent.DOM_VK_END:
-    aEvent.preventDefault(); // else the page scrolls unwantedly
-    break;
   }
 }
 
@@ -198,8 +190,7 @@ function restoreSingleTab(aIx, aShifted) {
   ss.setTabState(newTab, JSON.stringify(tabState));
 
   // respect the preference as to whether to select the tab (the Shift key inverses)
-  var prefBranch = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);
-  if (prefBranch.getBoolPref("browser.tabs.loadInBackground") != !aShifted)
+  if (Services.prefs.getBoolPref("browser.tabs.loadInBackground") != !aShifted)
     tabbrowser.selectedTab = newTab;
 }
 
