@@ -11,7 +11,6 @@ Components.utils.import("resource://gre/modules/LoginManagerContent.jsm");
 Components.utils.import("resource:///modules/Sanitizer.jsm");
 Components.utils.import("resource:///modules/mailnewsMigrator.js");
 
-var onContentLoaded = LoginManagerContent.onContentLoaded.bind(LoginManagerContent);
 var onFormPassword = LoginManagerContent.onFormPassword.bind(LoginManagerContent);
 var onUsernameInput = LoginManagerContent.onUsernameInput.bind(LoginManagerContent);
 
@@ -271,7 +270,6 @@ SuiteGlue.prototype = {
   },
 
   onStateChange: function(aWebProgress, aRequest, aStateFlags, aStatus) {
-    aWebProgress.DOMWindow.addEventListener("DOMContentLoaded", onContentLoaded, true);
     aWebProgress.DOMWindow.addEventListener("DOMFormHasPassword", onFormPassword, true);
     aWebProgress.DOMWindow.addEventListener("DOMAutoComplete", onUsernameInput, true);
     aWebProgress.DOMWindow.addEventListener("change", onUsernameInput, true);
